/*
Let's examine the impact of fine-grained access or 
virtual private database on the result cache.
*/

DROP PACKAGE frc_with_vpd
/

DROP TABLE employees_feuertip
/
CREATE TABLE employees_feuertip
AS SELECT * FROM employees
/

CREATE OR REPLACE PACKAGE frc_with_vpd 
IS
   FUNCTION last_name (employee_id_in IN employees_feuertip.employee_id%TYPE)
      RETURN employees_feuertip.last_name%TYPE
      result_cache;   
   
   FUNCTION restrict_employees_feuertip (schema_in VARCHAR2, NAME_IN VARCHAR2)
      RETURN VARCHAR2;
END;
/

CREATE OR REPLACE PACKAGE BODY frc_with_vpd
IS
   FUNCTION last_name (employee_id_in IN employees_feuertip.employee_id%TYPE)
      RETURN employees_feuertip.last_name%TYPE
      result_cache relies_on (employees_feuertip)
   IS
      onerow_rec   employees_feuertip%ROWTYPE;
   BEGIN
      DBMS_OUTPUT.PUT_LINE ( 'Looking up last name for employee ID ' || employee_id_in );
      SELECT *
        INTO onerow_rec
        FROM employees_feuertip
       WHERE employee_id = employee_id_in;

      RETURN onerow_rec.last_name;
   END;   
   
   FUNCTION restrict_employees_feuertip (schema_in VARCHAR2, NAME_IN VARCHAR2)
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN (CASE USER
                 WHEN 'SFEUERSTEIN'
                    THEN '1 = 1'
                 ELSE '1 = 2'
              END
             );
   END restrict_employees_feuertip;
END;
/

GRANT SELECT ON employees_feuertip TO sfeuerstein2
/
GRANT EXECUTE ON frc_with_vpd TO sfeuerstein2
/

BEGIN
   BEGIN
      sys.DBMS_RLS.drop_policy ('SFEUERSTEIN', 'employees_feuertip', 'rls_and_rc');
   EXCEPTION
      WHEN OTHERS
      THEN
         IF SQLCODE = -28102
         THEN
            NULL;
         ELSE
            RAISE;
         END IF;
   END;

   DBMS_RLS.add_policy
                    (object_schema        => 'SFEUERSTEIN'
                   , object_name          => 'employees_feuertip'
                   , policy_name          => 'rls_and_rc'
                   , function_schema      => 'SFEUERSTEIN'
                   , policy_function      => 'frc_with_vpd.restrict_employees_feuertip'
                   , statement_types      => 'SELECT,UPDATE,DELETE,INSERT'
                   , update_check         => TRUE
                    );
END;
/

/*
First demonstrate basic effect of VPD. Run in sfeuerstein
*/

CONNECT SFEUERSTEIN/SFEUERSTEIN@oracle11

SELECT last_name
  FROM employees_feuertip
 WHERE employee_id = 198
/

CONNECT sfeuerstein2/tiger@oracle11

SELECT last_name
  FROM SFEUERSTEIN.employees_feuertip
 WHERE employee_id = 198
/

CONNECT SFEUERSTEIN/SFEUERSTEIN@oracle11

SET SERVEROUTPUT ON

BEGIN
   /* Should see OConnell */
   DBMS_OUTPUT.put_line (frc_with_vpd.last_name (198));
END;
/

CONNECT sfeuerstein2/tiger@oracle11

SET SERVEROUTPUT ON

BEGIN
   /* Should raise NO_DATA_FOUND but instead.... */
   DBMS_OUTPUT.put_line (SFEUERSTEIN.frc_with_vpd.last_name (198));
END;
/

CONNECT SFEUERSTEIN/SFEUERSTEIN@oracle11

SET SERVEROUTPUT ON

BEGIN
   /* Flush the cache */
   UPDATE employees_feuertip
      SET last_name = SUBSTR (last_name, 1, 500);

   COMMIT;
END;
/

CONNECT sfeuerstein2/tiger@oracle11

SET SERVEROUTPUT ON

BEGIN
   /* Should raise NO_DATA_FOUND */
   DBMS_OUTPUT.put_line (SFEUERSTEIN.frc_with_vpd.last_name (198));
END;
/