DECLARE
  v_element JSON_ELEMENT_T;
BEGIN
  -- JSON is all about KEY:VALUE pairs
  -- JSON_ELEMENT_T deals with the VALUES on the right hand
  -- side of a KEY:VALUE pair such as this array
  v_element := JSON_ELEMENT_T.PARSE('[1,2,3,4,5]');
  -- It can also contain more KEY:VALUE pairs...
  v_element := JSON_ELEMENT_T.PARSE('{"Name":"Marty"}');
  v_element := JSON_ELEMENT_T.PARSE('[{"A":"B"},{"B":"C"}]');
  v_element := JSON_ELEMENT_T.PARSE('{"Name":{"FName":"Marty"}}');
END;
/

DECLARE
  v_element JSON_ELEMENT_T;
BEGIN
  -- The JSON must be valid though...
  v_element := JSON_ELEMENT_T.PARSE('Not JSON At All');
END;
/

DECLARE
  v_element JSON_ELEMENT_T;
      PROCEDURE what_am_i(p_id NUMBER) IS
      BEGIN
        DBMS_OUTPUT.PUT_LINE('#' || p_id);
        DBMS_OUTPUT.PUT_LINE('Size ' || v_element.GET_SIZE);
        IF v_element.IS_OBJECT THEN
          DBMS_OUTPUT.PUT_LINE('OBJECT');
        END IF;
        IF v_element.IS_ARRAY THEN
          DBMS_OUTPUT.PUT_LINE('ARRAY');
        END IF;
      END;
BEGIN
  -- It can interrogate itself...
  v_element := JSON_ELEMENT_T.PARSE('{"Name":"Marty"}');
  what_am_i(1);
  v_element := JSON_ELEMENT_T.PARSE('[1,2,3,4,5]');
  what_am_i(2);
  v_element := JSON_ELEMENT_T.PARSE('[{"A":"B"},{"B":"C"}]');
  what_am_i(3);
END;
/

DECLARE
  v_element JSON_ELEMENT_T;
BEGIN
  -- JSON_ELEMENT can be forgiving...
  v_element := JSON_ELEMENT_T.PARSE('{"Name":"Marty"}');
  DBMS_OUTPUT.PUT_LINE(v_element.TO_STRING);
  DBMS_OUTPUT.PUT_LINE(v_element.TO_NUMBER);
END;
/

DECLARE
  v_element JSON_ELEMENT_T;
BEGIN
  -- JSON_ELEMENT can be unforgiving...
  -- 0 = no errors (default)
  -- 1 = errors
  -- 2 =
  -- 3 =
  v_element := JSON_ELEMENT_T.PARSE('{"Name":"Marty"}');
  v_element.ON_ERROR(1);
  DBMS_OUTPUT.PUT_LINE(v_element.TO_STRING);
  DBMS_OUTPUT.PUT_LINE(v_element.TO_NUMBER);
END;
/

DECLARE
  v_element JSON_ELEMENT_T;
BEGIN
  -- JSON_ELEMENT_T cannot GET values directly from an array
  v_element := JSON_ELEMENT_T.PARSE('[1,2,3,4,5]');
  DBMS_OUTPUT.PUT_LINE(v_element.get_number(1));
END;
/

DECLARE
  v_element JSON_ELEMENT_T;
BEGIN
  -- but it can stringify them
  v_element := JSON_ELEMENT_T.PARSE('[1,2,3,4,5]');
  DBMS_OUTPUT.PUT_LINE(v_element.stringify);
  DBMS_OUTPUT.PUT_LINE(v_element.to_string);
END;
/
