/* file on web: 12c_table_pf_with_aa.sql */

/* Create a package-based type */

CREATE OR REPLACE PACKAGE aa_pkg AUTHID DEFINER
IS
   TYPE record_t IS RECORD
   (
      nm    VARCHAR2 (100),
      sal   NUMBER
   );

   TYPE array_t IS TABLE OF record_t
      INDEX BY PLS_INTEGER;

   FUNCTION my_array
      RETURN array_t;
END;
/

CREATE OR REPLACE PACKAGE BODY aa_pkg
IS
   FUNCTION my_array
      RETURN array_t
   IS
      l_return   array_t;
   BEGIN
      l_return (1).nm := 'Me';
      l_return (1).sal := 1000;
      l_return (200).nm := 'You';
      l_return (200).sal := 2;

      RETURN l_return;
   END;
END;
/

DECLARE
   l_array   aa_pkg.array_t;
BEGIN
   l_array := aa_pkg.my_array;

   FOR rec IN (  SELECT *
                   FROM TABLE (l_array)
               ORDER BY nm)
   LOOP
      DBMS_OUTPUT.put_line (rec.nm);
   END LOOP;
END;
/

/* What if I want access to the index inside SQL?
   Need to "expose it" in an array somehow. */

CREATE OR REPLACE PACKAGE aa_pkg AUTHID DEFINER
IS
   TYPE record_t IS RECORD
   (
      idx   INTEGER,
      nm    VARCHAR2 (100),
      sal   NUMBER
   );

   TYPE array_t IS TABLE OF record_t
      INDEX BY PLS_INTEGER;

   FUNCTION my_array
      RETURN array_t;
END;
/

CREATE OR REPLACE PACKAGE BODY aa_pkg
IS
   FUNCTION my_array
      RETURN array_t
   IS
      l_return   array_t;
   BEGIN
      l_return (1).nm := 'Me';
      l_return (1).sal := 1000;
      l_return (-200).nm := 'You';
      l_return (-200).sal := 2;

      RETURN l_return;
   END;
END;
/

DECLARE
   l_array   aa_pkg.array_t;
   l_index   PLS_INTEGER;
BEGIN
   l_array := aa_pkg.my_array;

   l_index := l_array.FIRST;

   WHILE l_index IS NOT NULL
   LOOP
      l_array (l_index).idx := l_index;
      l_index := l_array.next (l_index);
   END LOOP;

   FOR rec IN (  SELECT *
                   FROM TABLE (l_array)
               ORDER BY idx)
   LOOP
      DBMS_OUTPUT.put_line (rec.idx || ' = ' || rec.nm);
   END LOOP;
END;
/