DROP TYPE animal_ot FORCE
/

DROP TYPE animals_nt FORCE
/

DROP TABLE animals
/

CREATE TABLE animals
(
   name VARCHAR2 (10),
   species VARCHAR2 (20),
   date_of_birth DATE
)
/

/* I want to do something like this */

INSERT INTO animals (name, species, date_of_birth)
SELECT * FROM TABLE (a_function (...))
/

/* So I'd love to be able to create a nested table as follows */

CREATE TYPE animals_nt IS TABLE OF animals%ROWTYPE;
/

/* But %ROWTYPE is a PL/SQL construct, not known in the SQL layer.
   So instead I must:
      1. Create an object type that mimics the table.
      2. Create a nested table or varry of those types.
*/      

CREATE TYPE animal_ot IS OBJECT
(
   name VARCHAR2 (10),
   species VARCHAR2 (20),
   date_of_birth DATE
);
/

CREATE TYPE animals_nt IS TABLE OF animal_ot;
/

CREATE OR REPLACE FUNCTION animal_family (dad_in IN animal_ot, mom_in IN animal_ot)
   RETURN animals_nt
   AUTHID DEFINER
IS
   l_family   animals_nt := animals_nt (dad_in, mom_in);
BEGIN
   FOR indx IN 1 ..
               CASE mom_in.species
                  WHEN 'RABBIT' THEN 12
                  WHEN 'DOG' THEN 4
                  WHEN 'KANGAROO' THEN 1
               END
   LOOP
      l_family.EXTEND;
      l_family (l_family.LAST) :=
         animal_ot ('BABY' || indx,
                 mom_in.species,
                 ADD_MONTHS (SYSDATE, -1 * DBMS_RANDOM.VALUE (1, 6)));
   END LOOP;

   RETURN l_family;
END;
/

SELECT name, species, date_of_birth
  FROM TABLE (
          animal_family (animal_ot ('Hoppy', 'RABBIT', SYSDATE - 500),
                         animal_ot ('Hippy', 'RABBIT', SYSDATE - 300)))
/   

/* And now I can also INSERT-SELECT from the table function! */

INSERT INTO animals
SELECT name, species, date_of_birth
  FROM TABLE (
          animal_family (animal_ot ('Hoppy', 'RABBIT', SYSDATE - 500),
                      animal_ot ('Hippy', 'RABBIT', SYSDATE - 300)))
/ 

REM Pipelined version

CREATE OR REPLACE FUNCTION animal_family_pl (dad_in IN animal_ot, mom_in IN animal_ot)
   RETURN animals_nt
   AUTHID DEFINER
   PIPELINED
IS
BEGIN
   PIPE ROW (dad_in);
   PIPE ROW (mom_in);

   FOR indx IN 1 ..
               CASE mom_in.species
                  WHEN 'RABBIT' THEN 12
                  WHEN 'DOG' THEN 4
                  WHEN 'KANGAROO' THEN 1
               END
   LOOP
      PIPE ROW (animal_ot ('BABY' || indx,
                        mom_in.species,
                        ADD_MONTHS (SYSDATE, -1 * DBMS_RANDOM.VALUE (1, 6))));
   END LOOP;

   RETURN;
END;
/

SELECT name, species, date_of_birth
  FROM TABLE (
          animal_family_pl (animal_ot ('Bob', 'KANGAROO', SYSDATE - 1000),
                            animal_ot ('Sally', 'KANGAROO', SYSDATE - 700)))
/                         