DROP TYPE ticker_ot FORCE;
DROP TYPE tickers_nt FORCE;
DROP TYPE stock_ot FORCE;
DROP TYPE stocks_nt FORCE;
DROP TABLE stocks;
DROP TABLE tickers;
DROP TABLE more_stocks;

CREATE TABLE stocks
(
   ticker        VARCHAR2 (20),
   trade_date    DATE,
   open_price    NUMBER,
   close_price   NUMBER
)
/

BEGIN
   FOR indx IN 1 .. 1000
   LOOP
      -- Might as well be optimistic!
      INSERT INTO stocks
           VALUES ('STK' || indx,
                   SYSDATE,
                   indx,
                   indx + 15);
   END LOOP;

   COMMIT;
END;
/

CREATE TABLE tickers
(
   ticker      VARCHAR2 (20),
   pricedate   DATE,
   pricetype   VARCHAR2 (1),
   price       NUMBER
)
/

CREATE TYPE ticker_ot AUTHID DEFINER IS OBJECT 
(
   ticker VARCHAR2 (20),
   pricedate DATE,
   pricetype VARCHAR2 (1),
   price NUMBER
);
/

CREATE TYPE tickers_nt AS TABLE OF ticker_ot;
/

CREATE OR REPLACE PACKAGE stock_mgr AUTHID DEFINER
IS
   TYPE stocks_rc IS REF CURSOR RETURN stocks%ROWTYPE;

   TYPE tickers_rc IS REF CURSOR RETURN tickers%ROWTYPE;
END stock_mgr;
/

CREATE OR REPLACE FUNCTION doubled (rows_in stock_mgr.stocks_rc)
   RETURN tickers_nt
   AUTHID DEFINER
IS
   TYPE stocks_aat IS TABLE OF stocks%ROWTYPE INDEX BY PLS_INTEGER;
   l_stocks    stocks_aat;

   l_doubled   tickers_nt := tickers_nt ();
BEGIN
   LOOP
      FETCH rows_in BULK COLLECT INTO l_stocks LIMIT 100;
      EXIT WHEN l_stocks.COUNT = 0;

      FOR l_row IN 1 .. l_stocks.COUNT
      LOOP
         l_doubled.EXTEND;
         l_doubled (l_doubled.LAST) :=
            ticker_ot (l_stocks (l_row).ticker,
                       l_stocks (l_row).trade_date,
                       'O',
                       l_stocks (l_row).open_price);

         l_doubled.EXTEND;
         l_doubled (l_doubled.LAST) :=
            ticker_ot (l_stocks (l_row).ticker,
                       l_stocks (l_row).trade_date,
                       'C',
                       l_stocks (l_row).close_price);
      END LOOP;
   END LOOP;
   CLOSE rows_in;

   RETURN l_doubled;
END;
/

INSERT INTO tickers
   SELECT * 
     FROM TABLE (doubled (CURSOR (SELECT * FROM stocks)))
/

SELECT COUNT (*) FROM tickers
/

     SELECT *
       FROM tickers
FETCH FIRST 10 ROWS ONLY
/

CREATE OR REPLACE TYPE stock_ot AUTHID DEFINER IS OBJECT 
(
   ticker VARCHAR2 (20),
   trade_date DATE,
   open_price NUMBER,
   close_price NUMBER
)
/

CREATE OR REPLACE TYPE stocks_nt AS TABLE OF stock_ot;
/

CREATE OR REPLACE FUNCTION singled (tickers_in IN stock_mgr.tickers_rc)
   RETURN stocks_nt
   AUTHID DEFINER
IS
   TYPE tickers_aat IS TABLE OF tickers%ROWTYPE
      INDEX BY PLS_INTEGER;

   l_tickers    tickers_aat;

   l_singles        stocks_nt := stocks_nt ();
BEGIN
   LOOP
      FETCH tickers_in BULK COLLECT INTO l_tickers LIMIT 100;

      EXIT WHEN l_tickers.COUNT = 0;
      
      FOR indx IN 1 .. l_tickers.COUNT
      LOOP
         l_singles.EXTEND;
         l_singles (l_singles.LAST) :=
            stock_ot (l_tickers (indx).ticker,
                       l_tickers (indx).pricedate,
                       l_tickers (indx).price,
                       l_tickers (indx).price * .5);
      END LOOP;
   END LOOP;

   RETURN l_singles;
END;
/

/* CTAS - create a new table from two step transformation */

CREATE TABLE more_stocks
AS
   SELECT *
     FROM TABLE (
             singled (
                CURSOR (
                   SELECT * 
                     FROM TABLE (doubled (
                                   CURSOR (SELECT * FROM stocks))))))
/

SELECT COUNT (*)
  FROM more_stocks
/