/* Implicit cursor */

FUNCTION player_id (
   comp_event_id_in IN INTEGER, user_id_in IN INTEGER)
   RETURN qdb_compev_players.compev_player_id%TYPE
IS
   l_return   qdb_compev_players.compev_player_id%TYPE;
BEGIN
   SELECT cp.compev_player_id
     INTO l_return
     FROM qdb_compev_players cp
    WHERE cp.comp_event_id = player_id.comp_event_id_in 
      AND cp.user_id = player_id.user_id_in;

   RETURN l_return;
EXCEPTION
   WHEN NO_DATA_FOUND
   THEN
      RETURN NULL;
END;

/* Explicit cursor */

FUNCTION player_id (comp_event_id_in IN INTEGER, user_id_in IN INTEGER)
   RETURN qdb_compev_players.compev_player_id%TYPE
IS
   CURSOR cur
   IS
      SELECT cp.compev_player_id
        FROM qdb_compev_players cp
       WHERE     cp.comp_event_id = player_id.comp_event_id_in
             AND cp.user_id = player_id.user_id_in;

   l_return   qdb_compev_players.compev_player_id%TYPE;
BEGIN
   OPEN cur;

   FETCH cur INTO l_return;

   CLOSE cur;

   RETURN l_return;
END;

/* Cursor FOR Loop for single row */

FUNCTION player_id (comp_event_id_in IN INTEGER, user_id_in IN INTEGER)
   RETURN qdb_compev_players.compev_player_id%TYPE
IS
   l_return   qdb_compev_players.compev_player_id%TYPE;
BEGIN
   FOR rec
      IN (SELECT cp.compev_player_id
            FROM qdb_compev_players cp
           WHERE     cp.comp_event_id = player_id.comp_event_id_in
                 AND cp.user_id = player_id.user_id_in)
   LOOP
      l_return := rec.compev_player_id;
   END LOOP;

   RETURN l_return;
END;

/* Execute immediate - ONLY IF NECESSARY! */

FUNCTION player_id (
   comp_event_id_in IN INTEGER, user_id_in IN INTEGER)
   RETURN qdb_compev_players.compev_player_id%TYPE
IS
   c_query   CONSTANT VARCHAR2 (32000) := 
  'SELECT cp.compev_player_id
     FROM qdb_compev_players cp
    WHERE cp.comp_event_id = :ce_id 
      AND cp.user_id = :u_id';

   l_return  qdb_compev_players.compev_player_id%TYPE;
BEGIN
   EXECUTE IMMEDIATE c_query 
      INTO l_return
     USING comp_event_id_in, user_id_in;

   RETURN l_return;
EXCEPTION
   WHEN NO_DATA_FOUND
   THEN
      RETURN NULL;
END;



