-- grant select on sys.v_$result_cache_objects to test_user;
-- grant select on sys.v_$result_cache_dependency to test_user;

CREATE OR REPLACE PROCEDURE show_frc_dependencies (
   name_like_in   IN VARCHAR2
 ,  action_in      IN VARCHAR2 DEFAULT NULL)
IS
BEGIN
   DBMS_OUTPUT.put_line (RPAD ('-', 80, '-'));

   DBMS_OUTPUT.put_line (
         CASE
            WHEN action_in IS NOT NULL THEN action_in || ': '
            ELSE NULL
         END
      || 'Dependencies for "'
      || name_like_in
      || '"');

   FOR rec
      IN (SELECT d.result_id
               ,  TRANSLATE (
                     SUBSTR (res.name, 1, INSTR (res.name, ':') - 1)
                   ,  'A"'
                   ,  'A')
                     function_name
               ,  dep.name depends_on
            FROM v$result_cache_dependency d
               ,  v$result_cache_objects res
               ,  v$result_cache_objects dep
           WHERE     res.id = d.result_id
                 AND dep.id = d.depend_id
                 AND res.name LIKE name_like_in)
   LOOP
      /* Do not include dependency on self */
      IF rec.function_name <> rec.depends_on
      THEN
         DBMS_OUTPUT.put_line (
            rec.function_name || ' depends on ' || rec.depends_on);
      END IF;
   END LOOP;

   DBMS_OUTPUT.put_line (RPAD ('-', 80, '-'));
END;
/

CREATE TABLE qz_a (col VARCHAR2 (2));

CREATE TABLE qz_b (col VARCHAR2 (2));

INSERT INTO qz_a
     VALUES ('a1');

INSERT INTO qz_b
     VALUES ('b1');

COMMIT;

CREATE OR REPLACE FUNCTION qz_result_cache
   RETURN VARCHAR2
   RESULT_CACHE
IS
   res   VARCHAR2 (2);
BEGIN
   DBMS_OUTPUT.put_line ('Hello!');

   SELECT col INTO res FROM qz_a;

   IF res = 'a2'
   THEN
      SELECT col INTO res FROM qz_b;
   END IF;

   RETURN res;
END;
/

DECLARE
   s   VARCHAR2 (100);
BEGIN
   s := qz_result_cache;
   show_frc_dependencies ('%qz_RESULT_CACHE%');
END;
/

DECLARE
   s   VARCHAR2 (100);
BEGIN
   s := qz_result_cache;

   UPDATE qz_b
      SET col = 'b2';

   COMMIT;
   s :=qz_result_cache;
   show_frc_dependencies ('%qz_RESULT_CACHE%');
END;
/

DECLARE
   s   VARCHAR2 (100);
BEGIN
   s := qz_result_cache;

   UPDATE qz_a
      SET col = 'a2';

   COMMIT;
   s :=qz_result_cache;
   show_frc_dependencies ('%qz_RESULT_CACHE%');
END;
/

DROP TABLE qz_a PURGE;
DROP TABLE qz_b PURGE;
DROP FUNCTION qz_result_cache;