CREATE OR REPLACE FUNCTION tst_func (p IN VARCHAR2)
   RETURN VARCHAR2
IS
BEGIN
   RETURN rtrim (p || ' ');
END;
/

CREATE OR REPLACE PROCEDURE tst_proc (p IN OUT VARCHAR2)
IS
BEGIN
   p := rtrim (p || ' ');
END;
/

CREATE OR REPLACE PROCEDURE tst_proc_nocopy (p IN OUT NOCOPY VARCHAR2)
IS
BEGIN
   p := rtrim (p || ' ');
END;
/


CREATE OR REPLACE PROCEDURE run_test (string_len INTEGER, iter INTEGER)
IS
   t   NUMBER;

   v   VARCHAR2 (32767);

   PROCEDURE putresults (start_time NUMBER, unitname VARCHAR2)
   IS
   BEGIN
      DBMS_OUTPUT.put_line (
            'Took: '
         || RPAD ( (DBMS_UTILITY.get_cpu_time - start_time) / 100, 6)
         || 'sec. for '
         || unitname);
   END;
BEGIN
   v := RPAD (' ', string_len);

   DBMS_OUTPUT.put_line (
      'Running test with ' || LENGTH (v) || ' VARCHAR2 variable.');

   t := DBMS_UTILITY.get_time;

   FOR i IN 1 .. iter
   LOOP
      v := tst_func (v);
   END LOOP;

   putresults (t, 'v:= tst_func(v);');

   t := DBMS_UTILITY.get_cpu_time;

   FOR i IN 1 .. iter
   LOOP
      tst_proc (v);
   END LOOP;

   putresults (t, 'tst_proc(v);');

   t := DBMS_UTILITY.get_cpu_time;

   FOR i IN 1 .. iter
   LOOP
      tst_proc_nocopy (v);
   END LOOP;

   putresults (t, 'tst_proc_nocopy(v);');
END;
/

DECLARE
   iter   CONSTANT INTEGER := 10000000;
BEGIN
   DBMS_OUTPUT.put_line ('Running test with ' || iter || ' repetitive calls');

   run_test (32767, iter);

   run_test (3277, iter);

   run_test (327, iter);

   run_test (32, iter);

   run_test (3, iter);

   run_test (1, iter);
END;
/