CREATE TABLE qz_things
(
   thing_name      VARCHAR2 (100),
   thing_special   VARCHAR2 (100)
)
/

BEGIN
   INSERT INTO qz_things (thing_name, thing_special)
        VALUES ('Bird', 'Flies through the air');

   INSERT INTO qz_things (thing_name, thing_special)
        VALUES ('Earthworm', 'Breaks up soil');

   INSERT INTO qz_things (thing_name, thing_special)
        VALUES ('Screen', 'Stops bugs');

   COMMIT;
END;
/

CREATE OR REPLACE PACKAGE qz_thing_mgr
   AUTHID DEFINER
IS
   CURSOR thing_cur
   IS
        SELECT thing_name || ' ' || thing_special || '!' description
          FROM qz_things
      ORDER BY thing_name;

   TYPE desc_nt IS TABLE OF thing_cur%ROWTYPE;
END;
/

CREATE OR REPLACE PROCEDURE qz_show_count_and_data
   AUTHID DEFINER
AS
   l_count   PLS_INTEGER;
BEGIN
   SELECT COUNT (*) INTO l_count FROM qz_things;

   DBMS_OUTPUT.put_line (l_count || ' Things');

   FOR thing_rec IN qz_thing_mgr.thing_cur
   LOOP
      DBMS_OUTPUT.put_line (thing_rec.description);
   END LOOP;
END;
/

BEGIN
   qz_show_count_and_data;
END;
/

CREATE OR REPLACE PROCEDURE qz_show_count_and_data
   AUTHID DEFINER
AS
   l_descriptions   qz_thing_mgr.desc_nt;
BEGIN
   OPEN qz_thing_mgr.thing_cur;

   FETCH qz_thing_mgr.thing_cur BULK COLLECT INTO l_descriptions;

   CLOSE qz_thing_mgr.thing_cur;

   DBMS_OUTPUT.put_line (l_descriptions.COUNT || ' Things');

   FOR indx IN 1 .. l_descriptions.COUNT
   LOOP
      DBMS_OUTPUT.put_line (l_descriptions (indx).description);
   END LOOP;
END;
/

BEGIN
   qz_show_count_and_data;
END;
/

CREATE OR REPLACE PROCEDURE qz_show_count_and_data
   AUTHID DEFINER
AS
   l_first_line   BOOLEAN := TRUE;
BEGIN
   FOR thing_rec
      IN (  SELECT COUNT (*) OVER () row_count,
                   thing_name || ' ' || thing_special || '!' description
              FROM qz_things
          ORDER BY thing_name)
   LOOP
      IF l_first_line
      THEN
         DBMS_OUTPUT.put_line (thing_rec.row_count || ' Things');

         l_first_line := FALSE;
      END IF;

      DBMS_OUTPUT.put_line (thing_rec.description);
   END LOOP;
END;
/

BEGIN
   qz_show_count_and_data;
END;
/

CREATE OR REPLACE PROCEDURE qz_show_count_and_data
   AUTHID DEFINER
AS
   c_query   CONSTANT VARCHAR2 (1000)
      := q'[SELECT thing_name || ' ' || thing_special || '!' description
       FROM qz_things ORDER BY thing_name]' ;

   l_count            PLS_INTEGER;
   l_description      VARCHAR2 (1000);
BEGIN
   SELECT COUNT (*) INTO l_count FROM qz_things;

   EXECUTE IMMEDIATE 'SELECT COUNT(*) FROM (' || c_query || ')' INTO l_count;

   DBMS_OUTPUT.put_line (l_count || 'Things');

   OPEN c_query;

   LOOP
      FETCH c_query INTO l_description;

      EXIT WHEN c_query%NOTFOUND;

      DBMS_OUTPUT.put_line (l_description);
   END LOOP;

   CLOSE c_query;
END;
/

BEGIN
   qz_show_count_and_data;
END;
/