CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   /* Lots of code executing and then... */
   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      NULL;
END;
/

BEGIN my_procedure (100); END;
/

/* If you really want to ignore an error, then be specific */

CREATE OR REPLACE PROCEDURE my_procedure (name_in IN VARCHAR2)
   AUTHID DEFINER
IS
BEGIN
   EXECUTE IMMEDIATE 'CREATE TABLE ' || name_in || ' (n number)';
EXCEPTION
   WHEN OTHERS
   THEN
      IF SQLCODE = -955
      THEN
         /* ORA-00955: name is already used by an existing object */
         NULL;
      ELSE
         RAISE;
      END IF;
END;
/

BEGIN my_procedure ('NEW!!TABLE'); END;
/

BEGIN my_procedure ('EMPLOYEES'); END;
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   /* Lots of code executing and then... */
   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      RAISE;
END;
/

BEGIN my_procedure (100); END;
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   /* Lots of code executing and then... */
   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line ('Program failed!');
END;
/

BEGIN my_procedure (100); END;
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   /* Lots of code executing and then... */
   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line (SQLERRM);
END;
/

BEGIN my_procedure (100); END;
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   /* Lots of code executing and then... */
   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line (DBMS_UTILITY.format_error_stack);
END;
/

BEGIN my_procedure (100); END;
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   /* Lots of code executing and then... */
   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line (DBMS_UTILITY.format_error_stack);
      RAISE;
END;
/

BEGIN my_procedure (100); END;
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in     IN     INTEGER,
                                          status_out      OUT INTEGER)
   AUTHID DEFINER
IS
BEGIN
   IF value_in > 100
   THEN
      /* All is fine */
      /* Execute some code */
      /* Set status to "ok" */
      status_out := 0;
   ELSE
      RAISE PROGRAM_ERROR;
   END IF;
EXCEPTION
   WHEN OTHERS
   THEN
      status_out := SQLCODE;
END;
/

DECLARE
   l_status INTEGER;
BEGIN 
   my_procedure (100, l_status); 

   IF l_status <> 0
   THEN
      /* That didn't go well. Need to stop or do *something*! */
      RAISE program_error;
   END IF;
   
   my_procedure (110, l_status);
   
   IF l_status <> 0
   THEN
      /* That didn't go well. Need to stop or do *something*! */
      RAISE program_error;
   END IF;
END;
/

DROP TABLE error_log
/

CREATE TABLE error_log
(
   id               NUMBER GENERATED ALWAYS AS IDENTITY PRIMARY KEY,
   title            VARCHAR2 (200),
   info             CLOB,
   created_on       DATE DEFAULT SYSDATE,
   created_by       VARCHAR2 (100),
   callstack        CLOB,
   errorstack       CLOB,
   errorbacktrace   CLOB
)
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      INSERT INTO error_log (title,
                             info,
                             created_by,
                             callstack,
                             errorstack,
                             errorbacktrace)
           VALUES ('Program failed',
                   'value_in = ' || value_in,
                   USER,
                   DBMS_UTILITY.format_call_stack,
                   DBMS_UTILITY.format_error_stack,
                   DBMS_UTILITY.format_error_backtrace);

      RAISE;
END;
/

BEGIN my_procedure (100); END;
/

SELECT title, info FROM error_log
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   SAVEPOINT my_transaction;

   UPDATE employees
      SET salary = value_in;

   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      ROLLBACK TO my_transaction;

      INSERT INTO error_log (title,
                             info,
                             created_by,
                             callstack,
                             errorstack,
                             errorbacktrace)
           VALUES ('Program failed',
                   'value_in = ' || value_in,
                   USER,
                   DBMS_UTILITY.format_call_stack,
                   DBMS_UTILITY.format_error_stack,
                   DBMS_UTILITY.format_error_backtrace);

      SAVEPOINT error_logged;

      RAISE; 
END;
/

BEGIN my_procedure (100); END;
/

SELECT title, info FROM error_log
/

CREATE OR REPLACE PROCEDURE log_error (title_in   IN error_log.title%TYPE,
                                       info_in    IN error_log.info%TYPE)
   AUTHID DEFINER
IS
   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
   INSERT INTO error_log (title,
                          info,
                          created_by,
                          callstack,
                          errorstack,
                          errorbacktrace)
        VALUES (title_in,
                info_in,
                USER,
                DBMS_UTILITY.format_call_stack,
                DBMS_UTILITY.format_error_stack,
                DBMS_UTILITY.format_error_backtrace);

   COMMIT;
END;
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
   l_local_variable   DATE;
BEGIN
   l_local_variable :=
      CASE WHEN value_in > 100 THEN SYSDATE - 10 ELSE SYSDATE + 10 END;

   UPDATE employees
      SET salary = value_in;

   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      log_error (
         'my_procedure failed',
            'value_in = '
         || value_in
         || 'l_local_variable = '
         || TO_CHAR (l_local_variable, 'YYYY-MM-DD HH24:MI:SS'));

      RAISE;
END;
/

BEGIN my_procedure (100); END;
/

SELECT title, info, errorbacktrace 
  FROM error_log
/

ALTER SESSION SET PLSQL_WARNINGS = 'ENABLE:ALL'
/

CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
BEGIN
   /* Lots of code executing and then... */
   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      NULL;
END;
/

/* With logger */
CREATE OR REPLACE PROCEDURE my_procedure (value_in IN INTEGER)
   AUTHID DEFINER
IS
   l_local_variable   DATE;

   /* For logger parameter tracing */
   l_tab_param        logger.tab_param;
BEGIN
   logger.append_param (l_tab_param, 'value_in', value_in);

   l_local_variable :=
      CASE WHEN value_in > 100 THEN SYSDATE - 10 ELSE SYSDATE + 10 END;

   UPDATE employees
      SET salary = value_in;

   RAISE PROGRAM_ERROR;
EXCEPTION
   WHEN OTHERS
   THEN
      logger.log_error (
         p_scope    => $$plsql_unit,
         p_text     => 'Unexpected Failure',
         p_params   => l_tab_param,
         p_extra    =>    'l_local_variable = '
                       || TO_CHAR (l_local_variable, 'YYYY-MM-DD HH24:MI:SS'));
      RAISE;
END;
/

BEGIN my_procedure (100); END;
/

SELECT text, scope, extra
  FROM logger_logs
/

