CREATE TYPE food_ot AS OBJECT
(
   name VARCHAR2 (100),
   food_group VARCHAR2 (50),
   grown_in VARCHAR2 (100)
)
   NOT FINAL;
/

/* If object types were like records.... */

DECLARE
   TYPE food_rt IS RECORD
   (
      name         VARCHAR2 (100),
      food_group   VARCHAR2 (50),
      grown_in     VARCHAR2 (100)
   );

   my_favorite_vegetable   food_rt;
BEGIN
   my_favorite_vegetable.name := 'Brussels Sprouts';
   my_favorite_vegetable.food_group := 'Vegetables';
   my_favorite_vegetable.grown_in := 'Dirt';
   DBMS_OUTPUT.put_line (my_favorite_vegetable.name || ' are yummy!');
END;
/

DECLARE
   my_favorite_vegetable   food_ot;
BEGIN
   my_favorite_vegetable.name := 'Brussels Sprouts';
   my_favorite_vegetable.food_group := 'Vegetables';
   my_favorite_vegetable.grown_in := 'Dirt';
   DBMS_OUTPUT.put_line (my_favorite_vegetable.name || ' are yummy!');
END;
/

/* Must instantiate with constructor */

DECLARE
   my_favorite_vegetable   food_ot
      := food_ot ('Brussels Sprouts', 'Vegetables', 'Dirt');
BEGIN
   DBMS_OUTPUT.put_line (my_favorite_vegetable.name || ' are yummy!');
END;
/

/* Use your own constructor */

CREATE OR REPLACE TYPE food_ot AS OBJECT
(
   name VARCHAR2 (100),
   food_group VARCHAR2 (50),
   grown_in VARCHAR2 (100),
   CONSTRUCTOR FUNCTION food_ot
      RETURN SELF AS RESULT,
   CONSTRUCTOR FUNCTION food_ot (name_in IN VARCHAR2)
      RETURN SELF AS RESULT
)
   NOT FINAL;
/

CREATE OR REPLACE TYPE BODY food_ot
IS
   CONSTRUCTOR FUNCTION food_ot
      RETURN SELF AS RESULT
   IS
   BEGIN
      RETURN;
   END;

   CONSTRUCTOR FUNCTION food_ot (name_in IN VARCHAR2)
      RETURN SELF AS RESULT
   IS
   BEGIN
      self.name := name_in;
      RETURN;
   END;
END;
/

DECLARE
   my_favorite_vegetable   food_ot := food_ot ('Brussels Sprouts');
BEGIN
   DBMS_OUTPUT.put_line (my_favorite_vegetable.name || ' are yummy!');
END;
/

DECLARE
   my_favorite_vegetable   food_ot := food_ot ();
BEGIN
   my_favorite_vegetable.name := 'Brussels Sprouts';
   DBMS_OUTPUT.put_line (my_favorite_vegetable.name || ' are yummy!');
END;
/

DECLARE
   my_favorite_vegetable   food_ot
      := food_ot ('Brussels Sprouts', 'Vegetables', 'Dirt');
BEGIN
   DBMS_OUTPUT.put_line (my_favorite_vegetable.name || ' are yummy!');
END;
/

/* Overload default constructor */

CREATE OR REPLACE TYPE food_ot
   AS OBJECT
(
   name VARCHAR2 (100),
   food_group VARCHAR2 (50),
   grown_in VARCHAR2 (100),
   CONSTRUCTOR FUNCTION food_ot (name_in         IN VARCHAR2,
                                 food_group_in   IN VARCHAR2,
                                 grown_in_in     IN VARCHAR2)
      RETURN SELF AS RESULT
)
   NOT FINAL;
/

CREATE OR REPLACE TYPE BODY food_ot
IS
   CONSTRUCTOR FUNCTION food_ot (name_in         IN VARCHAR2,
                                 food_group_in   IN VARCHAR2,
                                 grown_in_in     IN VARCHAR2)
      RETURN SELF AS RESULT
   IS
   BEGIN
      self.name := name_in;
      self.food_group := grown_in_in;
      self.grown_in := food_group_in;
      RETURN;
   END;
END;
/

DECLARE
   my_favorite_vegetable   food_ot
      := food_ot ('Brussels Sprouts', 'Vegetables', 'Dirt');
BEGIN
   DBMS_OUTPUT.put_line (
         my_favorite_vegetable.name
      || ' are grown in '
      || my_favorite_vegetable.grown_in);
END;
/

/* PLS-00307: too many declarations of 'FOOD_OT' match this call */

DECLARE
   my_favorite_vegetable   food_ot
      := food_ot (name_in         => 'Brussels Sprouts',
                  food_group_in   => 'Vegetables',
                  grown_in_in     => 'Dirt');
BEGIN
   DBMS_OUTPUT.put_line (
         my_favorite_vegetable.name
      || ' are grown in '
      || my_favorite_vegetable.grown_in);
END;
/

DECLARE
   my_favorite_vegetable   food_ot
      := food_ot (name         => 'Brussels Sprouts',
                  food_group   => 'Vegetables',
                  grown_in     => 'Dirt');
BEGIN
   DBMS_OUTPUT.put_line (
         my_favorite_vegetable.name
      || ' are grown in '
      || my_favorite_vegetable.grown_in);
END;
/


CREATE OR REPLACE TYPE food_ot
   AS OBJECT
(
   name VARCHAR2 (100),
   food_group VARCHAR2 (50),
   grown_in VARCHAR2 (100),
   CONSTRUCTOR FUNCTION food_ot (name         IN VARCHAR2,
                                 food_group   IN VARCHAR2,
                                 grown_in     IN VARCHAR2)
      RETURN SELF AS RESULT
)
   NOT FINAL;
/

/* PLS-00330: invalid use of type name or subtype name */

CREATE OR REPLACE TYPE BODY food_ot
IS
   CONSTRUCTOR FUNCTION food_ot (name         IN VARCHAR2,
                                 food_group   IN VARCHAR2,
                                 grown_in     IN VARCHAR2)
      RETURN SELF AS RESULT
   IS
   BEGIN
      self.name := food_ot.name;
      self.food_group := food_ot.grown_in;
      self.grown_in := food_ot.food_group;
      RETURN;
   END;
END;
/

CREATE OR REPLACE TYPE BODY food_ot
IS
   CONSTRUCTOR FUNCTION food_ot (name         IN VARCHAR2,
                                 food_group   IN VARCHAR2,
                                 grown_in     IN VARCHAR2)
      RETURN SELF AS RESULT
   IS
   BEGIN
      self.name := name;
      self.food_group := grown_in;
      self.grown_in := food_group;
      RETURN;
   END;
END;
/

DECLARE
   my_favorite_vegetable   food_ot
      := food_ot (name         => 'Brussels Sprouts',
                  food_group   => 'Vegetables',
                  grown_in     => 'Dirt');
BEGIN
   DBMS_OUTPUT.put_line (
         my_favorite_vegetable.name
      || ' are grown in '
      || my_favorite_vegetable.grown_in);
END;
/

DECLARE
   my_favorite_vegetable   food_ot
      := food_ot ('Brussels Sprouts', 'Vegetables', 'Dirt');
BEGIN
   DBMS_OUTPUT.put_line (
         my_favorite_vegetable.name
      || ' are grown in '
      || my_favorite_vegetable.grown_in);
END;
/