CREATE OR REPLACE PACKAGE BODY ov_external
IS
   /* Created by: Steven Feuerstein
      Collects together all functionality needed to serve up
      Content Hub data externally: topics, resources
   */
   FUNCTION get_version
      RETURN VARCHAR2
   IS
   BEGIN
      -- ANY MAJOR CHANGES THAT EFFECTS OTHER PACKAGES, INCREASE 1ST NUMBER 01.00.00 to 02.00.00
      -- ANY PACKAGE SPEC CHANGES, INCREASE 2ND NUMBER 01.00.00 to 01.01.00
      -- ANY MINOR CHANGES OR BUG FIXES, INCREASE 3RD NUMBER 01.00.00 to 01.00.01
      -- 1.00.00 20160520 sfeuerstein initial version.
      -- 1.00.01 20160620 sfeuerstein removed get_First_attachment - moved to PLCH code base
      -- 1.00.02 20170614 sfeuerstein change references from ov_category_versions to ov_category_all_versions_v
      RETURN '1.00.02';
   END get_version;

   /* Utility functions */

   FUNCTION category_id_from_name (category_name_i   IN key_lookup_string_t,
                                   ignore_ndf_i      IN VARCHAR2 DEFAULT 'Y')
      RETURN ov_categories.category_id%TYPE
   IS
      l_category_id   ov_categories.category_id%TYPE;
   BEGIN
      SELECT c.category_id
        INTO l_category_id
        FROM ov_categories c
       WHERE c.category_name = category_name_i;

      RETURN l_category_id;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         CASE ignore_ndf_i
            WHEN 'Y'
            THEN
               RETURN NULL;
            ELSE
               RAISE;
         END CASE;
   END;

   FUNCTION version_from_names (category_i     IN key_lookup_string_t,
                                version_i      IN key_lookup_string_t,
                                ignore_ndf_i   IN VARCHAR2 DEFAULT 'Y')
      RETURN INTEGER
   IS
      l_version_id   INTEGER;
   BEGIN
      SELECT version_id
        INTO l_version_id
        FROM ov_categories c, ov_category_all_versions_v v
       WHERE     c.category_name = category_i
             AND c.category_id = v.category_id
             AND v.version_name = version_i;

      RETURN l_version_id;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         CASE ignore_ndf_i
            WHEN 'Y'
            THEN
               RETURN NULL;
            ELSE
               RAISE;
         END CASE;
   END;

   FUNCTION name_from_version (
      version_id_i   IN ov_category_all_versions_v.version_id%TYPE)
      RETURN VARCHAR2
   IS
      l_version_name   ov_category_all_versions_v.version_name%TYPE;
   BEGIN
      SELECT version_name
        INTO l_version_name
        FROM ov_category_all_versions_v v
       WHERE v.version_id = version_id_i;

      RETURN l_version_name;
   END;

   FUNCTION source_from_name (source_i       IN key_lookup_string_t,
                              ignore_ndf_i   IN VARCHAR2 DEFAULT 'Y')
      RETURN INTEGER
   IS
      l_source_id   INTEGER;
   BEGIN
      SELECT source_id
        INTO l_source_id
        FROM ov_sources s
       WHERE s.source_name = source_i;

      RETURN l_source_id;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         CASE ignore_ndf_i
            WHEN 'Y'
            THEN
               RETURN NULL;
            ELSE
               RAISE;
         END CASE;
   END;

   FUNCTION one_resource (resource_id_i IN ov_resources.resource_id%TYPE)
      RETURN ov.ov_resources_external%ROWTYPE
   IS
      l_return   ov.ov_resources_external%ROWTYPE;
   BEGIN
      SELECT *
        INTO l_return
        FROM ov.ov_resources_external
       WHERE resource_id = resource_id_i;

      RETURN l_return;
   END;

   /* Dn't thinnk this will be needed.
   FUNCTION resources_for_topic (
      topic_id_i              IN ov_topic_resources.topic_id%TYPE,
      usage_i                 IN VARCHAR2 DEFAULT NULL,
      source_i                IN VARCHAR2 DEFAULT NULL,
      resource_order_i        IN VARCHAR2 DEFAULT NULL,
      all_resources_below_i   IN VARCHAR2 DEFAULT 'Y')
      RETURN ov_resources_t
   IS
      l_resources   ov_resources_t;

      l_query       VARCHAR2 (32767)
         :=    'SELECT * FROM ov.ov_resources_external';
      l_order_by    VARCHAR2 (100) := 'tr.resource_display_order';
   BEGIN
      IF source_i IS NOT NULL
      THEN
         l_query :=
               l_query
            || ' AND r.source_id = s.source_id AND s.source_name = '''
            || source_i
            || '''';
      ELSE
         l_query := l_query || ' AND r.source_id = s.source_id(+)';
      END IF;

      /* TODO SF This will not work until difficulty added to resource and then
                 we have to convert the value to an order. *-/
      IF resource_order_i IS NOT NULL
      THEN
         l_order_by :=
            CASE resource_order_i
               WHEN 'DIFFICULTY' THEN 'r.resource_difficulty, ' || l_order_by
            END;
      END IF;

      EXECUTE IMMEDIATE
            l_query
         || CASE all_resources_below_i
               WHEN 'Y'
               THEN
                  ' AND t.topic_ID IN (SELECT topic_id FROM qdb_topics t2
START WITH t2.topic_id = :topic_id_i
CONNECT BY t2.parent_topic_id = PRIOR t2.topic_id)'
               ELSE
                  ' AND topic_ID = :topic_id_i '
            END
         || ' ORDER BY '
         || l_order_by
         BULK COLLECT INTO l_resources
         USING topic_id_i;

      RETURN l_resources;
   EXCEPTION
      WHEN OTHERS
      THEN
         logger.log_error (
            p_text    => 'Unhandled Exception',
            p_scope   => $$plsql_unit || '.resources_for_topic_External',
            p_extra   => 'Query = ' || l_query);
         RAISE;
   END;
   */

   PROCEDURE resolve_names_to_ids (
      source_i                IN     key_lookup_string_t,
      source_id_o                OUT INTEGER,
      approved_by_user_i      IN     key_lookup_string_t,
      approved_by_user_id_o      OUT INTEGER,
      author_i                IN     key_lookup_string_t,
      author_id_o                OUT INTEGER,
      submitter_i             IN     key_lookup_string_t,
      submitter_id_o             OUT INTEGER,
      created_by_i            IN     key_lookup_string_t,
      created_by_o               OUT INTEGER,
      usage_i                 IN     key_lookup_string_t,
      usage_id_o                 OUT INTEGER,
      language_i              IN     key_lookup_string_t,
      language_id_o              OUT INTEGER,
      content_type_i          IN     key_lookup_string_t,
      content_type_id_o          OUT INTEGER,
      resource_type_i         IN     key_lookup_string_t,
      resource_type_id_o         OUT INTEGER,
      approval_status_i       IN     key_lookup_string_t,
      approval_status_id_o       OUT INTEGER,
      change_status_i         IN     key_lookup_string_t,
      change_status_id_o         OUT INTEGER,
      failures_o                 OUT VARCHAR2)
   IS
   BEGIN
      IF source_i IS NOT NULL
      THEN
         source_id_o := source_from_name (source_i => source_i);

         IF source_id_o IS NULL
         THEN
            failures_o :=
               failures_o || ' Unable to retrieve source ID for ' || source_i;
         END IF;
      END IF;

      /* Users */

      IF approved_by_user_i IS NOT NULL
      THEN
         approved_by_user_id_o :=
            ov_utils.user_id (email_i                  => approved_by_user_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');
      END IF;

      IF author_i IS NOT NULL
      THEN
         author_id_o :=
            ov_utils.user_id (email_i                  => author_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');
      END IF;

      IF submitter_i IS NOT NULL
      THEN
         submitter_id_o :=
            ov_utils.user_id (email_i                  => submitter_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');
      END IF;

      IF created_by_i IS NOT NULL
      THEN
         created_by_o :=
            ov_utils.user_id (email_i                  => created_by_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');
      END IF;

      /* Types and Status */
      language_id_o :=
         ov_utils.lov_value_id (lov_name_i     => 'LANGUAGE',
                                lov_value1_i   => language_i);
      content_type_id_o :=
         ov_utils.lov_value_id (lov_name_i     => 'RESOURCE_CONTENT_TYPE',
                                lov_value1_i   => content_type_i);
      resource_type_id_o :=
         ov_utils.lov_value_id (lov_name_i     => 'RESOURCE_TYPE',
                                lov_value1_i   => resource_type_i);
      approval_status_id_o :=
         ov_utils.lov_value_id (lov_name_i     => 'RESOURCE_APPROVAL_STATUS',
                                lov_value1_i   => approval_status_i);
      change_status_id_o :=
         ov_utils.lov_value_id (lov_name_i     => 'RESOURCE_CHANGE_STATUS',
                                lov_value1_i   => change_status_i);

      /* Different source */
      /* No access
      BEGIN
         SELECT type_id
           INTO content_type_id_o
           FROM oll.ll_content_types
          WHERE name = content_type_i;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            failures_o :=
                  failures_o
               || ' Unable to retrieve content type ID for '
               || content_type_i;
      END;
      */
      usage_id_o :=
         CASE usage_i
            WHEN 'Documentation' THEN 32
            WHEN 'Whitepaper' THEN 6
            WHEN 'Presentation' THEN 14
            WHEN 'Web-based Utility' THEN 50
            WHEN 'Utility' THEN 50
            WHEN 'Article/Blog Posting' THEN 4
            WHEN 'Bug Report' THEN 39
            WHEN 'Code' THEN 42
            WHEN 'QUestion and Answer' THEN 9
            WHEN 'Tutorial' THEN 3
            WHEN 'Demonstration Script' THEN 42
            WHEN 'Tip/Tweet' THEN 49
            WHEN 'Quiz' THEN 31
            WHEN 'Video' THEN 8
            ELSE                                                      /* Link */
                29
         END;
   END;

   /* PROCEDURE ins
      1.02 String not FKY based API */

   PROCEDURE ins_resource (
      resource_name_i          IN     ov_resources.resource_name%TYPE,
      resource_description_i   IN     ov_resources.resource_description%TYPE,
      url_i                    IN     ov_resources.url%TYPE,
      resource_type_i          IN     key_lookup_string_t,
      content_type_i           IN     key_lookup_string_t,
      usage_i                  IN     key_lookup_string_t,
      source_i                 IN     key_lookup_string_t DEFAULT NULL,
      approval_status_i        IN     key_lookup_string_t DEFAULT NULL,
      change_status_i          IN     key_lookup_string_t DEFAULT NULL,
      approved_on_i            IN     ov_resources.approved_on%TYPE DEFAULT NULL,
      approved_by_user_i       IN     key_lookup_string_t DEFAULT NULL,
      author_i                 IN     key_lookup_string_t DEFAULT NULL,
      submitter_i              IN     key_lookup_string_t DEFAULT NULL,
      created_by_i             IN     key_lookup_string_t DEFAULT NULL,
      language_i               IN     key_lookup_string_t DEFAULT NULL,
      published_on_i           IN     ov_resources.published_on%TYPE DEFAULT NULL,
      available_on_i           IN     ov_resources.available_on%TYPE DEFAULT NULL,
      resource_sequence_i      IN     ov_resources.resource_sequence%TYPE DEFAULT NULL,
      resource_id_o               OUT ov_resources.resource_id%TYPE,
      source_name_i            IN     ov_resources.source_name%TYPE DEFAULT 'SOLVER',
      duration_mins_i          IN     ov_resources.duration_mins%TYPE DEFAULT NULL,
      remote_key_i             IN     ov_resources.remote_key%TYPE DEFAULT NULL,
      difficulty_i             IN     key_lookup_string_t DEFAULT NULL,
      minimum_version_id_i     IN     ov_category_all_versions_v.version_id%TYPE DEFAULT NULL,
      maximum_version_id_i     IN     ov_category_all_versions_v.version_id%TYPE DEFAULT NULL)
   IS
      l_tab_param   logger.tab_param;

      l_failure     VARCHAR2 (32767);
      l_resource    ov_resources%ROWTYPE;

      PROCEDURE initialize
      IS
      BEGIN
         logger.append_param (l_tab_param, 'resource_name_i', resource_name_i);
         logger.append_param (l_tab_param, 'url_i', url_i);
         logger.append_param (l_tab_param, 'resource_type_i', resource_type_i);
         logger.append_param (l_tab_param, 'content_type_i', content_type_i);
         logger.append_param (l_tab_param, 'usage_i', usage_i);
      END;
   BEGIN
      initialize;

      resolve_names_to_ids (source_i,
                            l_resource.source_id,
                            approved_by_user_i,
                            l_resource.approved_by_user_id,
                            author_i,
                            l_resource.author_id,
                            submitter_i,
                            l_resource.submitter_id,
                            created_by_i,
                            l_resource.created_by,
                            usage_i,
                            l_resource.usage_id,
                            language_i,
                            l_resource.language_id,
                            content_type_i,
                            l_resource.content_type_id,
                            resource_type_i,
                            l_resource.resource_type_id,
                            approval_status_i,
                            l_resource.approval_status_id,
                            change_status_i,
                            l_resource.change_status_id,
                            l_failure);

      IF l_failure IS NOT NULL
      THEN
         RAISE e_conversion_failure;
      END IF;

      ov.ov_resources_capi.ins (
         resource_name_i          => resource_name_i,
         resource_description_i   => resource_description_i,
         url_i                    => url_i,
         resource_type_id_i       => l_resource.resource_type_id,
         usage_id_i               => l_resource.usage_id,
         content_type_id_i        => l_resource.content_type_id,
         source_id_i              => l_resource.source_id,
         approval_status_id_i     => l_resource.approval_status_id,
         change_status_id_i       => l_resource.change_status_id,
         approved_on_i            => approved_on_i,
         approved_by_user_id_i    => l_resource.approved_by_user_id,
         author_id_i              => l_resource.author_id,
         submitter_id_i           => l_resource.submitter_id,
         created_by_i             => l_resource.created_by,
         language_id_i            => l_resource.language_id,
         published_on_i           => published_on_i,
         available_on_i           => available_on_i,
         resource_sequence_i      => resource_sequence_i,
         enabled_flag_i           => 'Y',
         featured_yn_i            => 'N',
         resource_id_o            => resource_id_o,
         source_name_i            => source_name_i,
         duration_mins_i          => duration_mins_i,
         remote_key_i             => remote_key_i,
         resource_level_i         => difficulty_i,
         minimum_version_id_i     => minimum_version_id_i,
         maximum_version_id_i     => maximum_version_id_i);
   EXCEPTION
      WHEN e_conversion_failure
      THEN
         logger.log_error (
            p_text     => 'Name->ID conversion failure? ' || l_failure,
            p_scope    => $$plsql_unit || '.ins (with strings)',
            p_params   => l_tab_param);
      WHEN OTHERS
      THEN
         logger.log_error (
            p_text     => NULL,
            p_scope    => $$plsql_unit || '.ins (with strings) OTHER EXCEPTION',
            p_params   => l_tab_param);
         RAISE;
   END;

   /* String version of upd: this assumes that topic is not changing. In addition, assumes
      that topic ID was previously determined and now passed in. */

   PROCEDURE upd_resource (
      resource_id_i            IN ov_resources.resource_id%TYPE,
      resource_name_i          IN ov_resources.resource_name%TYPE,
      resource_description_i   IN ov_resources.resource_description%TYPE,
      url_i                    IN ov_resources.url%TYPE,
      resource_type_i          IN key_lookup_string_t DEFAULT NULL,
      content_type_i           IN key_lookup_string_t,
      usage_i                  IN key_lookup_string_t DEFAULT NULL,
      source_i                 IN key_lookup_string_t DEFAULT NULL,
      approval_status_i        IN key_lookup_string_t DEFAULT NULL,
      change_status_i          IN key_lookup_string_t DEFAULT NULL,
      approved_on_i            IN ov_resources.approved_on%TYPE DEFAULT NULL,
      approved_by_user_i       IN key_lookup_string_t DEFAULT NULL,
      author_i                 IN key_lookup_string_t DEFAULT NULL,
      submitter_i              IN key_lookup_string_t DEFAULT NULL,
      updated_by_i             IN key_lookup_string_t DEFAULT NULL,
      language_i               IN key_lookup_string_t DEFAULT NULL,
      published_on_i           IN ov_resources.published_on%TYPE DEFAULT NULL,
      available_on_i           IN ov_resources.available_on%TYPE DEFAULT NULL,
      resource_sequence_i      IN ov_resources.resource_sequence%TYPE DEFAULT NULL,
      duration_mins_i          IN ov_resources.duration_mins%TYPE DEFAULT NULL,
      remote_key_i             IN ov_resources.remote_key%TYPE DEFAULT NULL,
      difficulty_i             IN key_lookup_string_t DEFAULT NULL,
      minimum_version_id_i     IN ov_category_all_versions_v.version_id%TYPE DEFAULT NULL,
      maximum_version_id_i     IN ov_category_all_versions_v.version_id%TYPE DEFAULT NULL)
   IS
      l_tab_param   logger.tab_param;
      l_failure     VARCHAR2 (32767);
      l_resource    ov.ov_resources_capi.ov_resources_rec;

      PROCEDURE initialize
      IS
      BEGIN
         logger.append_param (l_tab_param, 'resource_id_i', resource_id_i);
         logger.append_param (l_tab_param, 'resource_name_i', resource_name_i);
         logger.append_param (l_tab_param, 'url_i', url_i);

         /* Must set all columns NOT set via this subprogram from the current record. */
         l_resource := ov.ov_resources_capi.one_row (resource_id_i);
      END;
   BEGIN
      initialize;

      resolve_names_to_ids (source_i,
                            l_resource.source_id,
                            approved_by_user_i,
                            l_resource.approved_by_user_id,
                            author_i,
                            l_resource.author_id,
                            submitter_i,
                            l_resource.submitter_id,
                            updated_by_i,
                            l_resource.created_by,
                            usage_i,
                            l_resource.usage_id,
                            language_i,
                            l_resource.language_id,
                            content_type_i,
                            l_resource.content_type_id,
                            resource_type_i,
                            l_resource.resource_type_id,
                            approval_status_i,
                            l_resource.approval_status_id,
                            change_status_i,
                            l_resource.change_status_id,
                            l_failure);

      IF l_failure IS NOT NULL
      THEN
         RAISE e_conversion_failure;
      END IF;

      l_resource.resource_name := resource_name_i;
      l_resource.resource_description := resource_description_i;
      l_resource.url := url_i;
      l_resource.approved_on := approved_on_i;
      l_resource.published_on := published_on_i;
      l_resource.available_on := available_on_i;
      l_resource.resource_sequence := resource_sequence_i;
      l_resource.duration_mins := duration_mins_i;
      l_resource.remote_key := remote_key_i;
      l_resource.resource_level := difficulty_i;
      l_resource.minimum_version_id := minimum_version_id_i;
      l_resource.maximum_version_id := maximum_version_id_i;

      ov.ov_resources_capi.upd (l_resource);
   EXCEPTION
      WHEN e_conversion_failure
      THEN
         logger.log_error (
            p_text     => 'Name->ID conversion failure? ' || l_failure,
            p_scope    => $$plsql_unit || '.upd (with strings)',
            p_params   => l_tab_param);
      WHEN OTHERS
      THEN
         logger.log_error (
            p_text     => NULL,
            p_scope    => $$plsql_unit || '.upd (with strings) OTHER EXCEPTION',
            p_params   => l_tab_param);
         RAISE;
   END;

   FUNCTION resource_id_from_names (category_i        IN key_lookup_string_t,
                                    resource_name_i   IN key_lookup_string_t,
                                    url_i             IN key_lookup_string_t,
                                    source_i          IN key_lookup_string_t)
      RETURN ov_resources.resource_id%TYPE
   IS
      l_resource_id   ov_resources.resource_id%TYPE;
      l_tab_param     logger.tab_param;

      PROCEDURE initialize
      IS
      BEGIN
         logger.append_param (l_tab_param, 'category_i', category_i);
         logger.append_param (l_tab_param, 'resource_name_i', resource_name_i);
         logger.append_param (l_tab_param, 'url_i', url_i);
         logger.append_param (l_tab_param, 'source_i', source_i);
      END;
   BEGIN
      initialize;

      SELECT r.resource_id
        INTO l_resource_id
        FROM ov_categories c,
             ov_resources r,
             ov_topic_resources tr,
             ov_topics t,
             ov_sources s
       WHERE     r.resource_id = tr.resource_id
             AND tr.primary_flag = 'Y'
             AND t.topic_id = tr.topic_id
             AND t.category_id = c.category_id
             AND c.category_name = category_i
             AND r.resource_name = resource_name_i
             AND r.source_id = s.source_id(+)
             AND s.source_name = source_i
             AND r.url = url_i;

      RETURN l_resource_id;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
      WHEN TOO_MANY_ROWS
      THEN
         logger.log_error (
            p_text     => 'More than one resource found',
            p_scope    => $$plsql_unit || '.resource_id_from_names',
            p_params   => l_tab_param);
         RAISE;
   END;

   FUNCTION resource_id_from_url (category_i   IN key_lookup_string_t,
                                  url_i        IN key_lookup_string_t)
      RETURN ov_resources.resource_id%TYPE
   IS
      l_resource_id   ov_resources.resource_id%TYPE;
   BEGIN
      /* Find primary topic for resource, get its domain/category */
      SELECT r.resource_id
        INTO l_resource_id
        FROM ov_categories c,
             ov_resources r,
             ov_topic_resources tr,
             ov_topics t
       WHERE     r.resource_id = tr.resource_id
             AND tr.primary_flag = 'Y'
             AND t.topic_id = tr.topic_id
             AND t.category_id = c.category_id
             AND c.category_name = category_i
             AND r.url = url_i;

      RETURN l_resource_id;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
   END;

   PROCEDURE attach_resource (
      resource_id_i        IN ov_resources.resource_id%TYPE,
      to_resource_id_i     IN ov_resources.resource_id%TYPE,
      processing_order_i   IN VARCHAR2 DEFAULT 'AFTER')
   IS
   BEGIN
      INSERT INTO ov_resources_related (primary_resource_id,
                                        related_resource_id,
                                        related_resource_order,
                                        processing_order)
           VALUES (to_resource_id_i,
                   resource_id_i,
                   1,
                   processing_order_i);
   EXCEPTION
      WHEN DUP_VAL_ON_INDEX
      THEN
         /* Already attached, so just update the processing point */
         UPDATE ov_resources_related
            SET processing_order = processing_order_i
          WHERE     primary_resource_id = to_resource_id_i
                AND related_resource_id = resource_id_i;
   END;

   PROCEDURE set_minmax_versions (
      resource_id_i       IN ov_resources.resource_id%TYPE,
      category_i          IN key_lookup_string_t,
      maximum_version_i   IN key_lookup_string_t,
      minimum_version_i   IN key_lookup_string_t)
   IS
      l_category_id          INTEGER;
      l_minimum_version_id   INTEGER;
      l_maximum_version_id   INTEGER;
   BEGIN
      l_category_id := category_id_from_name (category_i, 'Y');

      IF maximum_version_i IS NOT NULL
      THEN
         l_maximum_version_id :=
            version_from_names (category_i   => category_i,
                                version_i    => maximum_version_i);
      END IF;

      IF minimum_version_i IS NOT NULL
      THEN
         l_minimum_version_id :=
            version_from_names (category_i   => category_i,
                                version_i    => minimum_version_i);
      END IF;

      UPDATE ov_resources
         SET maximum_version_id = l_maximum_version_id,
             minimum_version_id = l_minimum_version_id
       WHERE resource_id = resource_id_i;
   END;

   FUNCTION source_id_for_name (source_name_i IN VARCHAR2)
      RETURN ov_sources.source_id%TYPE
   IS
      l_return   ov_sources.source_id%TYPE;
   BEGIN
      SELECT source_id
        INTO l_return
        FROM ov_sources
       WHERE UPPER (source_name) = UPPER (source_name_i);

      RETURN l_return;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
   END;

   PROCEDURE ins_source (source_name_i          IN VARCHAR2,
                         url_i                  IN VARCHAR2,
                         source_description_i   IN VARCHAR2,
                         author_i               IN VARCHAR2,
                         created_by_i           IN VARCHAR2,
                         source_type_i          IN VARCHAR2,
                         enabled_flag_i         IN VARCHAR2)
   IS
      l_source   ov_sources_tapi.ov_sources_rec;

      PROCEDURE resolve_names_to_ids
      IS
      BEGIN
         IF author_i IS NOT NULL
         THEN
            l_source.author_id :=
               ov_utils.user_id (email_i                  => author_i,
                                 ignore_ndf_i             => 'Y',
                                 create_if_not_exists_i   => 'Y');
         END IF;

         IF created_by_i IS NOT NULL
         THEN
            l_source.created_by :=
               ov_utils.user_id (email_i                  => created_by_i,
                                 ignore_ndf_i             => 'Y',
                                 create_if_not_exists_i   => 'Y');
         END IF;
      END;
   BEGIN
      IF source_id_for_name (source_name_i) IS NULL
      THEN
         resolve_names_to_ids;
         ov_sources_tapi.ins (
            source_name_i          => source_name_i,
            source_description_i   => source_description_i,
            url_i                  => url_i,
            author_id_i            => l_source.author_id,
            source_type_i          => ov_utils.lov_value_id (
                                        lov_name_i     => 'SOURCE_TYPE',
                                        lov_value1_i   => source_type_i),
            created_by_i           => l_source.created_by,
            enabled_flag_i         => enabled_flag_i);
      END IF;
   END;

   /* Topic-related functionality */


   /** PROCEDURE ins
   * ins procedure with string inputs for external use */

   PROCEDURE ins_topic (
      category_i             IN     key_lookup_string_t DEFAULT NULL,
      parent_topic_i         IN     key_lookup_string_t DEFAULT NULL,
      grandparent_topic_i    IN     key_lookup_string_t DEFAULT NULL,
      topic_name_i           IN     ov_topics.topic_name%TYPE DEFAULT NULL,
      description_i          IN     ov_topics.description%TYPE DEFAULT NULL,
      how_its_useful_i       IN     ov_topics.how_its_useful%TYPE DEFAULT NULL,
      owner_i                IN     key_lookup_string_t DEFAULT NULL,
      sibling_position_i     IN     ov_topics.sibling_position%TYPE DEFAULT NULL,
      minimum_version_i      IN     key_lookup_string_t DEFAULT NULL,
      maximum_version_i      IN     key_lookup_string_t DEFAULT NULL,
      created_by_i           IN     key_lookup_string_t DEFAULT NULL,
      approved_on_create_i   IN     BOOLEAN DEFAULT FALSE,
      source_name_i          IN     ov_topics.source_name%TYPE,
      topic_id_o                OUT ov_topics.topic_id%TYPE)
   IS
      l_tab_param   logger.tab_param;

      l_failure     VARCHAR2 (32767);
      l_topic       ov_topics%ROWTYPE;

      PROCEDURE initialize
      IS
      BEGIN
         logger.append_param (l_tab_param, 'category_i', category_i);
         logger.append_param (l_tab_param, 'topic_name_i', topic_name_i);
         logger.append_param (l_tab_param, 'parent_topic_i', parent_topic_i);
         logger.append_param (l_tab_param,
                              'grandparent_topic_i',
                              grandparent_topic_i);
      END;

      PROCEDURE resolve_names_to_ids
      IS
      BEGIN
         l_topic.category_id := category_id_from_name (category_i, 'Y');

         IF l_topic.category_id IS NULL
         THEN
            l_failure :=
                  l_failure
               || ' Unable to retrieve category ID for '
               || category_i;
         END IF;

         IF parent_topic_i IS NOT NULL
         THEN
            l_topic.parent_topic_id :=
               topic_id_from_names (category_i       => category_i,
                                    topic_i          => parent_topic_i,
                                    parent_topic_i   => grandparent_topic_i);

            IF l_topic.parent_topic_id IS NULL
            THEN
               l_failure :=
                     l_failure
                  || ' Unable to retrieve parent topic for '
                  || category_i
                  || '-'
                  || grandparent_topic_i
                  || '-'
                  || parent_topic_i;
            END IF;
         ELSE
            l_topic.parent_topic_id := NULL;
         END IF;

         IF maximum_version_i IS NOT NULL
         THEN
            l_topic.maximum_version_id :=
               version_from_names (category_i   => category_i,
                                   version_i    => maximum_version_i);

            IF l_topic.maximum_version_id IS NULL
            THEN
               l_failure :=
                     l_failure
                  || ' Unable to retrieve version ID for '
                  || category_i
                  || '-'
                  || maximum_version_i;
            END IF;
         END IF;

         IF minimum_version_i IS NOT NULL
         THEN
            l_topic.minimum_version_id :=
               version_from_names (category_i   => category_i,
                                   version_i    => minimum_version_i);

            IF l_topic.minimum_version_id IS NULL
            THEN
               l_failure :=
                     l_failure
                  || ' Unable to retrieve version ID for '
                  || category_i
                  || '-'
                  || minimum_version_i;
            END IF;
         END IF;

         /* Users */

         l_topic.owner_id :=
            ov_utils.user_id (email_i                  => owner_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');

         l_topic.created_by :=
            ov_utils.user_id (email_i                  => created_by_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');

         IF l_failure IS NOT NULL
         THEN
            RAISE e_conversion_failure;
         END IF;
      END;
   BEGIN
      initialize;

      resolve_names_to_ids;

      ov.ov_topics_capi.ins (
         category_id_i          => l_topic.category_id,
         parent_topic_id_i      => l_topic.parent_topic_id,
         topic_name_i           => topic_name_i,
         topic_type_id_i        => ov_utils.lov_value_id (
                                     lov_name_i     => 'OV_TOPIC_TYPE',
                                     lov_value1_i   => 'DEFAULT'),
         description_i          => description_i,
         how_its_useful_i       => how_its_useful_i,
         owner_id_i             => l_topic.owner_id,
         sibling_position_i     => sibling_position_i,
         minimum_version_id_i   => l_topic.minimum_version_id,
         maximum_version_id_i   => l_topic.maximum_version_id,
         created_by_i           => l_topic.created_by,
         approved_on_create_i   => approved_on_create_i,
         source_name_i          => source_name_i,
         topic_id_o             => l_topic.topic_id);
   EXCEPTION
      WHEN e_conversion_failure
      THEN
         logger.log_error (
            p_text     => 'Name->ID conversion failure? ' || l_failure,
            p_scope    => $$plsql_unit || '.ins (with strings)',
            p_params   => l_tab_param);
      WHEN OTHERS
      THEN
         logger.log_error (
            p_text     => NULL,
            p_scope    => $$plsql_unit || '.ins (with strings) OTHER EXCEPTION',
            p_params   => l_tab_param);
   END;

   /** PROCEDURE upd
        * upd procedure with string inputs for external use */

   PROCEDURE upd_topic (
      topic_id_i            IN ov_topics.topic_id%TYPE,
      category_i            IN key_lookup_string_t DEFAULT NULL,
      parent_topic_i        IN key_lookup_string_t DEFAULT NULL,
      grandparent_topic_i   IN key_lookup_string_t DEFAULT NULL,
      topic_name_i          IN ov_topics.topic_name%TYPE DEFAULT NULL,
      description_i         IN ov_topics.description%TYPE DEFAULT NULL,
      how_its_useful_i      IN ov_topics.how_its_useful%TYPE DEFAULT NULL,
      owner_i               IN key_lookup_string_t DEFAULT NULL,
      sibling_position_i    IN ov_topics.sibling_position%TYPE DEFAULT NULL,
      minimum_version_i     IN key_lookup_string_t DEFAULT NULL,
      maximum_version_i     IN key_lookup_string_t DEFAULT NULL,
      updated_by_i          IN key_lookup_string_t DEFAULT NULL,
      approved_flag_i       IN ov_topics.approved_flag%TYPE,
      approved_on_i         IN ov_topics.approved_on%TYPE,
      approved_by_user_i    IN key_lookup_string_t)
   IS
      l_tab_param   logger.tab_param;

      l_failure     VARCHAR2 (32767);
      l_topic       ov_topics%ROWTYPE;

      PROCEDURE initialize
      IS
      BEGIN
         logger.append_param (l_tab_param, 'topic_id_i', topic_id_i);
         logger.append_param (l_tab_param, 'category_i', category_i);
         logger.append_param (l_tab_param, 'topic_name_i', topic_name_i);
         logger.append_param (l_tab_param, 'parent_topic_i', parent_topic_i);
         logger.append_param (l_tab_param,
                              'grandparent_topic_i',
                              grandparent_topic_i);
      END;

      PROCEDURE resolve_names_to_ids
      IS
      BEGIN
         BEGIN
            SELECT c.category_id
              INTO l_topic.category_id
              FROM ov_categories c
             WHERE c.category_name = category_i;
         EXCEPTION
            WHEN NO_DATA_FOUND
            THEN
               l_failure :=
                     l_failure
                  || ' Unable to retrieve category for '
                  || category_i;
         END;

         IF parent_topic_i IS NOT NULL
         THEN
            l_topic.parent_topic_id :=
               topic_id_from_names (category_i       => category_i,
                                    topic_i          => parent_topic_i,
                                    parent_topic_i   => grandparent_topic_i);

            IF l_topic.parent_topic_id IS NULL
            THEN
               l_failure :=
                     l_failure
                  || ' Unable to retrieve parent topic for '
                  || category_i
                  || '-'
                  || grandparent_topic_i
                  || '-'
                  || parent_topic_i;
            END IF;
         ELSE
            l_topic.parent_topic_id := NULL;
         END IF;

         IF maximum_version_i IS NOT NULL
         THEN
            BEGIN
               SELECT version_id
                 INTO l_topic.maximum_version_id
                 FROM ov_category_all_versions_v v
                WHERE     v.category_id = l_topic.category_id
                      AND v.version_name = maximum_version_i;
            EXCEPTION
               WHEN NO_DATA_FOUND
               THEN
                  l_failure :=
                        l_failure
                     || ' Unable to retrieve version ID for '
                     || category_i
                     || '-'
                     || maximum_version_i;
            END;
         END IF;

         IF minimum_version_i IS NOT NULL
         THEN
            BEGIN
               SELECT v.version_id
                 INTO l_topic.minimum_version_id
                 FROM ov_category_all_versions_v v
                WHERE     v.category_id = l_topic.category_id
                      AND v.version_name = minimum_version_i;
            EXCEPTION
               WHEN NO_DATA_FOUND
               THEN
                  l_failure :=
                        l_failure
                     || ' Unable to retrieve version ID for '
                     || category_i
                     || '-'
                     || minimum_version_i;
            END;
         END IF;

         /* Users */

         l_topic.owner_id :=
            ov_utils.user_id (email_i                  => owner_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');

         l_topic.updated_by :=
            ov_utils.user_id (email_i                  => updated_by_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');

         l_topic.approved_by_user_id :=
            ov_utils.user_id (email_i                  => approved_by_user_i,
                              ignore_ndf_i             => 'Y',
                              create_if_not_exists_i   => 'Y');

         IF l_failure IS NOT NULL
         THEN
            RAISE e_conversion_failure;
         END IF;
      END;
   BEGIN
      initialize;

      resolve_names_to_ids;

      ov.ov_topics_capi.upd (
         topic_id_i              => topic_id_i,
         category_id_i           => l_topic.category_id,
         parent_topic_id_i       => l_topic.parent_topic_id,
         topic_name_i            => topic_name_i,
         description_i           => description_i,
         how_its_useful_i        => how_its_useful_i,
         topic_type_id_i         => ov_utils.lov_value_id (
                                      lov_name_i     => 'OV_TOPIC_TYPE',
                                      lov_value1_i   => 'DEFAULT'),
         owner_id_i              => l_topic.owner_id,
         ccms_keyword_i          => l_topic.ccms_keyword,
         sibling_position_i      => sibling_position_i,
         minimum_version_id_i    => l_topic.minimum_version_id,
         maximum_version_id_i    => l_topic.maximum_version_id,
         updated_by_i            => l_topic.updated_by,
         approved_flag_i         => approved_flag_i,
         approved_on_i           => approved_on_i,
         approved_by_user_id_i   => l_topic.approved_by_user_id);
   EXCEPTION
      WHEN e_conversion_failure
      THEN
         logger.log_error (
            p_text     => 'Name->ID conversion failure? ' || l_failure,
            p_scope    => $$plsql_unit || '.upd (with strings)',
            p_params   => l_tab_param);
      WHEN OTHERS
      THEN
         logger.log_error (
            p_text     => NULL,
            p_scope    => $$plsql_unit || '.upd (with strings) OTHER EXCEPTION',
            p_params   => l_tab_param);
         RAISE;
   END;

   /* Topic uniqueness is only within the parent topic. */
   FUNCTION topic_id_from_names (category_i       IN key_lookup_string_t,
                                 topic_i          IN key_lookup_string_t,
                                 parent_topic_i   IN key_lookup_string_t)
      RETURN ov_topics.topic_id%TYPE
   IS
      l_scope       VARCHAR2 (100) := 'topic_id_from_names';
      l_topic_id    ov_topics.topic_id%TYPE;
      l_tab_param   logger.tab_param;

      PROCEDURE initialize
      IS
      BEGIN
         logger.append_param (l_tab_param, 'category_i', category_i);
         logger.append_param (l_tab_param, 'topic_i', topic_i);
         logger.append_param (l_tab_param, 'parent_topic_i', parent_topic_i);
      END;
   BEGIN
      initialize;

      IF parent_topic_i IS NULL
      THEN
         /* Special treatment for root */
         SELECT t.topic_id
           INTO l_topic_id
           FROM ov_categories c, ov_topics t
          WHERE     c.category_name = category_i
                AND c.category_id = t.category_id
                AND t.topic_name = topic_i;
      ELSE
         SELECT t.topic_id
           INTO l_topic_id
           FROM ov_categories c, ov_topics t, ov_topics pt
          WHERE     c.category_name = category_i
                AND c.category_id = t.category_id
                AND t.topic_name = topic_i
                AND t.parent_topic_id = pt.topic_id
                AND c.category_id = pt.category_id
                AND pt.topic_name = parent_topic_i;
      END IF;

      RETURN l_topic_id;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
      WHEN OTHERS
      THEN
         logger.log_error (
            p_text     => NULL,
            p_scope    => $$plsql_unit || '.topic_id_from_names OTHER EXCEPTION',
            p_params   => l_tab_param);
         RAISE;
   END;

   FUNCTION root_topic_id (category_i IN key_lookup_string_t)
      RETURN ov_topics.topic_id%TYPE
   IS
   BEGIN
      RETURN topic_id_from_names (category_i       => category_i,
                                  topic_i          => category_i,
                                  parent_topic_i   => NULL);
   END;

   PROCEDURE remove_for_app_source (app_source_in   IN VARCHAR2,
                                    test_in         IN BOOLEAN DEFAULT TRUE)
   IS
   BEGIN
      /* Need to delete from top down in hierarchy to ensure parent topic
         IDs are in place. Always include the Root of the domain so
         we start at the top. */
      FOR rec IN (    SELECT LEVEL, t.*
                        FROM ov_topics t
                       WHERE t.source_name = app_source_in
                  CONNECT BY NOCYCLE PRIOR t.topic_id = t.parent_topic_id
                    ORDER BY LEVEL DESC)
      LOOP
         IF test_in
         THEN
            DBMS_OUTPUT.put_line (
                  'delete '
               || rec.LEVEL
               || ' - '
               || rec.topic_id
               || ' - '
               || rec.parent_topic_id
               || ' - '
               || rec.topic_name);
         ELSE
            BEGIN
               DELETE FROM ov_topics
                     WHERE topic_id = rec.topic_id;
            EXCEPTION
               WHEN OTHERS
               THEN
                  NULL;
            END;
         END IF;
      END LOOP;
   END;
END;
/