CREATE OR REPLACE PACKAGE nocopy_test
IS
   TYPE stuff_rt IS RECORD
   (
      s    VARCHAR2 (100),
      n1   NUMBER,
      n2   NUMBER
   );

   TYPE stuff_nt IS TABLE OF stuff_rt;

   PROCEDURE pass_by_value (stuff_in IN OUT stuff_nt);

   PROCEDURE pass_by_ref (stuff_in IN OUT NOCOPY stuff_nt);
   
   PROCEDURE compare_methods (num IN PLS_INTEGER);
END;
/

CREATE OR REPLACE PACKAGE BODY nocopy_test
IS
   PROCEDURE pass_by_value (stuff_in IN OUT stuff_nt)
   IS
   BEGIN
      FOR indx IN 1 .. stuff_in.COUNT
      LOOP
         stuff_in (indx).n1 := stuff_in (indx).n1 * 2;
      END LOOP;
   END;

   PROCEDURE pass_by_ref (stuff_in IN OUT NOCOPY stuff_nt)
   IS
   BEGIN
      FOR indx IN 1 .. stuff_in.COUNT
      LOOP
         stuff_in (indx).n1 := stuff_in (indx).n1 * 2;
      END LOOP;
   END;

   PROCEDURE compare_methods (num IN PLS_INTEGER)
   IS
      l_stuff   stuff_nt := stuff_nt();

      l_start   TIMESTAMP;

      PROCEDURE start_timer
      IS
      BEGIN
         l_start := SYSTIMESTAMP;
      END start_timer;

      PROCEDURE show_elapsed_time (message_in IN VARCHAR2)
      IS
      BEGIN
         DBMS_OUTPUT.put_line (
               CASE
                  WHEN message_in IS NULL THEN 'Completed in:'
                  ELSE '"' || message_in || '" completed in: '
               END
            || REGEXP_SUBSTR (SYSTIMESTAMP - l_start,
                              '([1-9][0-9:]*|0)\.\d{3}')
            || ' seconds');

         /* Reset timer */
         start_timer;
      END show_elapsed_time;

      PROCEDURE loadtab
      IS
      BEGIN
         l_stuff.DELETE;
         l_stuff.EXTEND (100000);

         FOR indx IN 1 .. 100000
         LOOP
            l_stuff (indx).s := indx;
            l_stuff (indx).n1 := indx;
            l_stuff (indx).n2 := indx;
         END LOOP;
      END;
   BEGIN
      loadtab;

      start_timer;

      FOR indx IN 1 .. num
      LOOP
         pass_by_value (l_stuff);
      END LOOP;

      show_elapsed_time ('By value ' || num);

      loadtab;

      start_timer;

      FOR indx IN 1 .. num
      LOOP
         pass_by_ref (l_stuff);
      END LOOP;

      show_elapsed_time ('By reference (NOCOPY) ' || num);
   END;
END;
/

BEGIN
   nocopy_test.compare_methods (500);
END;
/
