create or replace PROCEDURE          "SEARCH" (
    p_string  in varchar2 default null,
    tag     in varchar2 default null,
    file    in varchar2 default null,
    office  in varchar2 default null,
    oh      in varchar2 default null,
    p_debug   in varchar2 default 'NO',
    p_trace   in varchar2 default 'NO',
    p_session in varchar2 default null)
is
  b int := dbms_utility.get_time;
  l_question_id int;
  l_off_hrs boolean;
  l_office varchar2(100) := nvl(office,oh);
  l_class  number(20);
  l_parms  varchar2(1000);
  l_additional_parms int;
  l_eq int;
begin
  if tag is not null then
    begin
      select id
      into   l_question_id
      from   ate_submitted_questions
      where  url_tag = lower(tag);

      wwv_flow.show (
          p_flow_id      => '100',
          p_flow_step_id => '11',
          p_debug        => p_debug,
          p_arg_names    => wwv_flow_utilities.string_to_table2('P11_QUESTION_ID'),
          p_arg_values   => wwv_flow_utilities.string_to_table2(l_question_id),
          p_instance     => p_session,
          p_trace        => p_trace
      );
    exception
      when no_data_found then
        --
        -- it might the actual question ID
        --
        if replace(translate(tag,'0123456789','@@@@@@@@@@'),'@') is null then
          select id
          into   l_question_id
          from   ate_submitted_questions
          where  id = lower(tag);

          wwv_flow.show (
              p_flow_id      => '100',
              p_flow_step_id => '11',
              p_debug        => p_debug,
              p_arg_names    => wwv_flow_utilities.string_to_table2('P11_QUESTION_ID'),
              p_arg_values   => wwv_flow_utilities.string_to_table2(l_question_id),
              p_instance     => p_session,
              p_trace        => p_trace
          );
        else
          raise;
        end if;
    end;
  elsif file is not null then
    wwv_flow.show (
        p_flow_id      => '100',
        p_flow_step_id => '9',
        p_debug        => p_debug,
        p_arg_names    => wwv_flow_utilities.string_to_table2('IR[presentation-downloads-reg]_ROWFILTER'),
        p_arg_values   => wwv_flow_utilities.string_to_table2(file),
        p_clear_cache  => wwv_flow_utilities.string_to_table2('RIR'),
        p_instance     => p_session,
        p_trace        => p_trace
    );

  elsif l_office is not null then
    begin
      --
      -- we either passed just the class, or we passed class,more_parameters
      --
      l_additional_parms := instr(l_office,',');
      if l_additional_parms > 0 then
        l_class := to_number(substr(l_office,1,l_additional_parms-1));  --  eg class=1234
        l_parms := substr(l_office,l_additional_parms+1);               --  eg P551_INVITED:P551_BLAH=Y:X
        l_eq    := instr(l_parms,'=');
      else
        l_class := l_office;
        l_parms := null;
        l_eq    := 0;
      end if;


      execute immediate 'begin :1 := qdb_prod.dg_class_mgr.is_office_hours(:2); end;' using out l_off_hrs, in to_number(l_class);
    exception
      when others then
        l_off_hrs := false;
    end;

    if l_off_hrs then
      wwv_flow.show (
          p_flow_id      => '100',
          p_flow_step_id => '551',
          p_debug        => p_debug,
          p_arg_names    => case
                              when l_parms is null or l_eq = 0 then
                                wwv_flow_utilities.string_to_table2('P551_CLASS_ID')
                              else
                                wwv_flow_utilities.string_to_table2('P551_CLASS_ID:'||substr(l_parms,1,instr(l_parms,'=')-1),sep=>':')
                            end,
          p_arg_values   => case
                              when l_parms is null or l_eq = 0 then
                                wwv_flow_utilities.string_to_table2(l_class)
                              else
                                wwv_flow_utilities.string_to_table2(l_class||':'||substr(l_parms,instr(l_parms,'=')+1),sep=>':')
                            end,
          p_clear_cache  => wwv_flow_utilities.string_to_table2('RIR'),
          p_instance     => p_session,
          p_trace        => p_trace
      );
    else
      wwv_flow.show (
          p_flow_id      => '100',
          p_flow_step_id => '1',
          p_debug        => p_debug,
          p_arg_names    => wwv_flow_utilities.string_to_table2('P1_SEARCH_CRITERIA'),
          p_arg_values   => wwv_flow_utilities.string_to_table2(p_string),
          p_instance     => p_session,
          p_trace        => p_trace
      );
    end if;
  else
    wwv_flow.show (
        p_flow_id      => '100',
        p_flow_step_id => '1',
        p_debug        => p_debug,
        p_arg_names    => wwv_flow_utilities.string_to_table2('P1_SEARCH_CRITERIA'),
        p_arg_values   => wwv_flow_utilities.string_to_table2(p_string),
        p_instance     => p_session,
        p_trace        => p_trace
    );
  end if;

exception
  when no_data_found then
    wwv_flow.show (
        p_flow_id      => '100',
        p_flow_step_id => '1',
        p_debug        => p_debug,
        p_instance     => p_session,
        p_trace        => p_trace
    );
end;
/