CREATE OR REPLACE PACKAGE sf_tracker
/*
Overview: sf_tracker allows you to keep track of whether a
certain name has already been used within a particular list.

Besides hopefully being useful to you, it is a nice little 
demonstration of string-indexed associative arrays and
nested collections (collections within collections)

Author: Steven Feuerstein

*/
IS
   SUBTYPE maxvarchar2_t IS VARCHAR2 (32767);

   SUBTYPE list_name_t IS maxvarchar2_t;

   SUBTYPE value_string_t IS maxvarchar2_t;

   PROCEDURE clear_all_lists;

   PROCEDURE clear_list (list_name_in IN list_name_t);

   PROCEDURE create_list (
      list_name_in IN list_name_t
    , case_sensitive_in IN BOOLEAN DEFAULT FALSE
    , overwrite_in IN BOOLEAN DEFAULT TRUE
   );

   -- Is the string already in use?
   FUNCTION string_in_use (
      list_name_in IN list_name_t
    , value_string_in IN value_string_t
   )
      RETURN BOOLEAN;

   -- Mark this string as being used.
   PROCEDURE mark_as_used (
      list_name_in IN list_name_t
    , value_string_in IN value_string_t
   );
END sf_tracker;
/

