/* Return message associated with current error code */

begin
   raise too_many_rows;
exception
   when others then
      dbms_output.put_line(sqlerrm);
end;
/

/* Behavior of sqlerrm for different "kinds" of error codes */

declare
   l_codes dbms_sql.number_table := dbms_sql.number_table(
      0  => -1,
      1  => 100,
      2  => -1403,
      3  => -1422,
      4  => 1422,
      5  => 1,
      6  => -20000,
      7  => -21000,
      8  => -21002,
      9  => -50000
   );
begin
   for indx in l_codes.first..l_codes.last loop
      begin
         dbms_output.put_line('sqlerrm('||l_codes(indx)||') => ' || sqlerrm(l_codes(indx)));
      exception
         when others then
            dbms_output.put_line('Unable to call sqlerrm with code ' || l_codes(indx));
      end;
   end loop;
end;
/

/* When does Oracle stop returning "error number argument to raise_application_error of  is out of range"? */

declare
   l_code integer := -21000;
begin
   loop
      dbms_output.put_line('sqlerrm('||l_code||') => ' || sqlerrm(l_code));
      exit when sqlerrm(l_code) not like 
         '%error number argument to raise_application_error%out of range%';
      exit when l_code < -30000;
      l_code := l_code - 1;
   end loop;
end;
/

/* Error message vs error stack */

create or replace procedure show_errors is
begin
   dbms_output.put_line('-------SQLERRM-------------');
   dbms_output.put_line(length(sqlerrm));
   dbms_output.put_line(sqlerrm);
   dbms_output.put_line('-------FORMAT_ERROR_STACK--');
   dbms_output.put_line(length(dbms_utility.format_error_stack));
   dbms_output.put_line(dbms_utility.format_error_stack);
end;
/

declare
   procedure proc1 is
   begin
      raise no_data_found;
   end;
   
   procedure proc2 is
   begin
      proc1;
   exception
      when others then
         raise value_error;
   end;
   
   procedure proc3 is
   begin
      proc2;
   exception
      when others then
         raise program_error;
   end;
begin
   proc3;
exception
   when others then
      show_errors;
end;
/