CREATE OR REPLACE PACKAGE BODY alert_mgr
IS
   g_trc      BOOLEAN := FALSE;
   g_trc_to   VARCHAR2 (10) := 'SCREEN';

   PROCEDURE settrc (on_in IN BOOLEAN, send_to_in IN VARCHAR2 DEFAULT 'SCREEN')
   IS
   BEGIN
      g_trc := on_in;
      g_trc_to := send_to_in;
   END;

   PROCEDURE trc (val IN VARCHAR2)
   IS
   BEGIN
      IF g_trc
      THEN
         IF g_trc_to = 'SCREEN'
         THEN
            DBMS_OUTPUT.put_line (val);
         ELSE
            LOG (val);
         END IF;
      END IF;
   END;

   PROCEDURE trc (val IN BOOLEAN)
   IS
   BEGIN
      trc (
         CASE val WHEN TRUE THEN 'TRUE' WHEN FALSE THEN 'FALSE' ELSE 'NULL' END);
   END;

   PROCEDURE trc (str IN VARCHAR2, val IN BOOLEAN)
   IS
   BEGIN
      trc (
            str
         || ' - '
         || CASE val
               WHEN TRUE THEN 'TRUE'
               WHEN FALSE THEN 'FALSE'
               ELSE 'NULL'
            END);
   END;

   PROCEDURE pl (val IN VARCHAR2)
   IS
   BEGIN
      DBMS_OUTPUT.put_line (val);
   END;

   PROCEDURE bpl (val IN BOOLEAN)
   IS
   BEGIN
      pl (
         CASE val WHEN TRUE THEN 'TRUE' WHEN FALSE THEN 'FALSE' ELSE 'NULL' END);
   END;

   PROCEDURE bpl (str IN VARCHAR2, val IN BOOLEAN)
   IS
   BEGIN
      pl (
            str
         || ' - '
         || CASE val
               WHEN TRUE THEN 'TRUE'
               WHEN FALSE THEN 'FALSE'
               ELSE 'NULL'
            END);
   END;

   PROCEDURE LOG (info_in IN VARCHAR2)
   IS
      PRAGMA AUTONOMOUS_TRANSACTION;
   BEGIN
      INSERT INTO alert_log (info)
           VALUES (info_in);

      COMMIT;
   END;

   /* Job management */

   PROCEDURE start_check_alerts_job (every_n_minutes_in IN NUMBER DEFAULT 5)
   IS
   BEGIN
      DBMS_SCHEDULER.create_job (
         job_name          => c_job_name,
         job_type          => 'PLSQL_BLOCK',
         job_action        => 'BEGIN alert_mgr.check_alerts; END;',
         start_date        => SYSDATE,
         repeat_interval   =>    'FREQ=MINUTELY;INTERVAL='
                              || every_n_minutes_in
                              || ';',
         enabled           => FALSE);

      DBMS_SCHEDULER.set_attribute (name        => c_job_name,
                                    attribute   => 'logging_level',
                                    VALUE       => DBMS_SCHEDULER.logging_full);

      DBMS_SCHEDULER.enable (c_job_name);

      UPDATE alert_config
         SET job_wakeup_minutes = every_n_minutes_in;

      resume_checking;
   END;

   PROCEDURE change_check_frequency (every_n_minutes_in IN NUMBER)
   IS
   BEGIN
      DBMS_SCHEDULER.set_attribute (
         name        => c_job_name,
         attribute   => 'repeat_interval',
         VALUE       => 'FREQ=MINUTELY;INTERVAL=' || every_n_minutes_in || ';');

      UPDATE alert_config
         SET job_wakeup_minutes = every_n_minutes_in;
   END;

   PROCEDURE stop_checking
   IS
   BEGIN
      UPDATE alert_config
         SET keep_checking = 'N';
   END;

   PROCEDURE resume_checking
   IS
   BEGIN
      UPDATE alert_config
         SET keep_checking = 'Y';
   END;

   FUNCTION checking_for_alerts
      RETURN BOOLEAN
   IS
      l_keep_checking   alert_config.keep_checking%TYPE;
   BEGIN
      SELECT keep_checking INTO l_keep_checking FROM alert_config;

      RETURN CASE l_keep_checking WHEN 'Y' THEN TRUE ELSE FALSE END;
   END;

   PROCEDURE stop_check_alerts_job
   IS
   BEGIN
      DBMS_SCHEDULER.disable (c_job_name);
   END;

   FUNCTION check_alerts_job_running
      RETURN BOOLEAN
   IS
   BEGIN
      RETURN TRUE;
   END;

   FUNCTION check_alerts_job_running_yn
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN 'Y';
   END;

   PROCEDURE check_alerts_job_status
   IS
   BEGIN
      NULL;
   END;

   PROCEDURE register_email_proc (proc_name_in        IN VARCHAR2,
                                  workspace_name_in   IN VARCHAR2 DEFAULT NULL)
   IS
   BEGIN
      UPDATE alert_config
         SET send_email_proc_name = proc_name_in,
             apex_workspace_name = workspace_name_in;
   END;

   PROCEDURE register_user (email_address_in IN VARCHAR2, id_out OUT NUMBER)
   IS
      l_id   NUMBER;
   BEGIN
      INSERT INTO alert_users (email_address)
           VALUES (email_address_in)
        RETURNING id
             INTO id_out;
   EXCEPTION
      WHEN DUP_VAL_ON_INDEX
      THEN
         /* Already registered */
         NULL;
   END;

   PROCEDURE register_user (email_address_in IN VARCHAR2)
   IS
      l_id   NUMBER;
   BEGIN
      register_user (email_address_in, l_id);
   END;

   FUNCTION register_user (email_address_in IN VARCHAR2)
      RETURN NUMBER
   IS
      l_id   NUMBER;
   BEGIN
      register_user (email_address_in, l_id);
      RETURN l_id;
   END;

   PROCEDURE register_alert (
      alert_name_in             IN     VARCHAR2,
      description_in            IN     VARCHAR2 DEFAULT NULL,
      trigger_code_in           IN     VARCHAR2,
      trigger_type_in           IN     VARCHAR2,
      check_frequency_in        IN     VARCHAR2,
      recurrence_frequency_in   IN     VARCHAR2,
      ignore_failures_in        IN     VARCHAR2,
      action_type_in            IN     VARCHAR2 DEFAULT NULL,
      action_code_in            IN     VARCHAR2 DEFAULT NULL,
      send_email_to_in          IN     VARCHAR2 DEFAULT NULL,
      email_subject_in          IN     VARCHAR2 DEFAULT NULL,
      email_body_in             IN     CLOB DEFAULT NULL,
      email_query_in            IN     VARCHAR2 DEFAULT NULL,
      is_active_in              IN     VARCHAR2 DEFAULT 'N',
      alert_id_out                 OUT NUMBER)
   IS
      PROCEDURE replace_actions (id_in IN INTEGER)
      IS
      BEGIN
         IF action_type_in IS NOT NULL
         THEN
            DELETE FROM alert_actions
                  WHERE alert_id = id_in;

            INSERT INTO alert_actions (alert_id,
                                       action_type,
                                       action_code,
                                       send_email_to,
                                       email_subject,
                                       email_body,
                                       email_query)
                 VALUES (id_in,
                         action_type_in,
                         action_code_in,
                         send_email_to_in,
                         email_subject_in,
                         email_body_in,
                         email_query_in);
         END IF;
      END;
   BEGIN
      INSERT INTO alerts (alert_name,
                          description,
                          trigger_code,
                          trigger_type,
                          check_frequency,
                          recurrence_frequency,
                          ignore_failures,
                          is_active)
           VALUES (alert_name_in,
                   description_in,
                   trigger_code_in,
                   trigger_type_in,
                   check_frequency_in,
                   recurrence_frequency_in,
                   ignore_failures_in,
                   is_active_in)
        RETURNING id
             INTO alert_id_out;

      replace_actions (alert_id_out);
   EXCEPTION
      WHEN DUP_VAL_ON_INDEX
      THEN
            UPDATE alerts
               SET description = description_in,
                   trigger_code = trigger_code_in,
                   trigger_type = trigger_type_in,
                   check_frequency = check_frequency_in,
                   recurrence_frequency = recurrence_frequency_in,
                   ignore_failures = ignore_failures_in,
                   is_active = is_active_in
             WHERE alert_name = UPPER (alert_name_in)
         RETURNING id
              INTO alert_id_out;

         replace_actions (alert_id_out);
   END;

   PROCEDURE register_alert (
      alert_name_in             IN VARCHAR2,
      description_in            IN VARCHAR2 DEFAULT NULL,
      trigger_code_in           IN VARCHAR2,
      trigger_type_in           IN VARCHAR2,
      check_frequency_in        IN VARCHAR2,
      recurrence_frequency_in   IN VARCHAR2,
      ignore_failures_in        IN VARCHAR2,
      action_type_in            IN VARCHAR2 DEFAULT NULL,
      action_code_in            IN VARCHAR2 DEFAULT NULL,
      send_email_to_in          IN VARCHAR2 DEFAULT NULL,
      email_subject_in          IN VARCHAR2 DEFAULT NULL,
      email_body_in             IN CLOB DEFAULT NULL,
      email_query_in            IN VARCHAR2 DEFAULT NULL,
      is_active_in              IN VARCHAR2 DEFAULT 'N')
   IS
      alert_id_out   NUMBER;
   BEGIN
      register_alert (alert_name_in             => alert_name_in,
                      description_in            => description_in,
                      trigger_code_in           => trigger_code_in,
                      trigger_type_in           => trigger_type_in,
                      check_frequency_in        => check_frequency_in,
                      recurrence_frequency_in   => recurrence_frequency_in,
                      ignore_failures_in        => ignore_failures_in,
                      action_type_in            => action_type_in,
                      action_code_in            => action_code_in,
                      send_email_to_in          => send_email_to_in,
                      email_subject_in          => email_subject_in,
                      email_body_in             => email_body_in,
                      email_query_in            => email_query_in,
                      is_active_in              => is_active_in,
                      alert_id_out              => alert_id_out);
   END;

   PROCEDURE register_alert (
      alert_name_in             IN     VARCHAR2,
      description_in            IN     VARCHAR2 DEFAULT NULL,
      trigger_code_in           IN     VARCHAR2,
      trigger_type_in           IN     VARCHAR2,
      check_frequency_in        IN     VARCHAR2,
      recurrence_frequency_in   IN     VARCHAR2,
      ignore_failures_in        IN     VARCHAR2,
      actions_in                IN     actions_t,
      is_active_in              IN     VARCHAR2 DEFAULT 'N',
      alert_id_out                 OUT NUMBER)
   IS
   BEGIN
      register_alert (alert_name_in             => alert_name_in,
                      description_in            => description_in,
                      trigger_code_in           => trigger_code_in,
                      trigger_type_in           => trigger_type_in,
                      check_frequency_in        => check_frequency_in,
                      recurrence_frequency_in   => recurrence_frequency_in,
                      ignore_failures_in        => ignore_failures_in,
                      is_active_in              => is_active_in,
                      alert_id_out              => alert_id_out);

      DELETE FROM alert_actions
            WHERE alert_id = alert_id_out;

      FORALL indx IN actions_in.FIRST .. actions_in.LAST
         INSERT INTO alert_actions (alert_id,
                                    action_type,
                                    action_code,
                                    send_email_to,
                                    email_subject,
                                    email_body,
                                    email_query)
              VALUES (alert_id_out,
                      actions_in (indx).action_type,
                      actions_in (indx).action_code,
                      actions_in (indx).send_email_to,
                      actions_in (indx).email_subject,
                      actions_in (indx).email_body,
                      actions_in (indx).email_query);
   END;

   PROCEDURE register_alert (alert_name_in             IN VARCHAR2,
                             description_in            IN VARCHAR2 DEFAULT NULL,
                             trigger_code_in           IN VARCHAR2,
                             trigger_type_in           IN VARCHAR2,
                             check_frequency_in        IN VARCHAR2,
                             recurrence_frequency_in   IN VARCHAR2,
                             ignore_failures_in        IN VARCHAR2,
                             actions_in                IN actions_t,
                             is_active_in              IN VARCHAR2 DEFAULT 'N')
   IS
      alert_id_out   NUMBER;
   BEGIN
      register_alert (alert_name_in             => alert_name_in,
                      description_in            => description_in,
                      trigger_code_in           => trigger_code_in,
                      trigger_type_in           => trigger_type_in,
                      check_frequency_in        => check_frequency_in,
                      recurrence_frequency_in   => recurrence_frequency_in,
                      ignore_failures_in        => ignore_failures_in,
                      actions_in                => actions_in,
                      is_active_in              => is_active_in,
                      alert_id_out              => alert_id_out);
   END;

   PROCEDURE follow_alert (alert_id_in IN NUMBER, user_id_in IN NUMBER)
   IS
   BEGIN
      INSERT INTO alert_follows (alert_id, user_id)
           VALUES (alert_id_in, user_id_in);
   EXCEPTION
      WHEN DUP_VAL_ON_INDEX
      THEN
         /* Already registered */
         NULL;
   END;

   PROCEDURE follow_alert (alert_id_in IN NUMBER, email_address_in IN VARCHAR2)
   IS
      l_user_id   alert_users.id%TYPE;
   BEGIN
      SELECT id
        INTO l_user_id
        FROM alert_users
       WHERE email_address = UPPER (email_address_in);

      follow_alert (alert_id_in, l_user_id);
   END;

   PROCEDURE follow_alert (alert_name_in      IN VARCHAR2,
                           email_address_in   IN VARCHAR2)
   IS
      l_user_id    alert_users.id%TYPE;
      l_alert_id   alerts.id%TYPE;
   BEGIN
      SELECT u.id, a.id
        INTO l_user_id, l_alert_id
        FROM alert_users u, alerts a
       WHERE     u.email_address = UPPER (email_address_in)
             AND a.alert_name = UPPER (alert_name_in);

      follow_alert (l_alert_id, l_user_id);
   END;

   PROCEDURE unfollow_alert (alert_id_in IN NUMBER, user_id_in IN NUMBER)
   IS
   BEGIN
      DELETE FROM alert_follows
            WHERE alert_id = alert_id_in AND user_id = user_id_in;
   END;

   PROCEDURE unfollow_alert (alert_name_in      IN VARCHAR2,
                             email_address_in   IN VARCHAR2)
   IS
      l_user_id    alert_users.id%TYPE;
      l_alert_id   alerts.id%TYPE;
   BEGIN
      SELECT u.id, a.id
        INTO l_user_id, l_alert_id
        FROM alert_users u, alerts a
       WHERE     u.email_address = UPPER (email_address_in)
             AND a.alert_name = UPPER (alert_name_in);

      unfollow_alert (l_alert_id, l_user_id);
   END;

   PROCEDURE activate_alert (id_in IN NUMBER)
   IS
   BEGIN
      UPDATE alerts
         SET is_active = 'Y'
       WHERE id = id_in;
   END;

   PROCEDURE deactivate_alert (id_in IN NUMBER)
   IS
   BEGIN
      UPDATE alerts
         SET is_active = 'N'
       WHERE id = id_in;
   END;

   FUNCTION time_to_trigger_alert (check_frequency_in          IN VARCHAR2,
                                   recurring_frequency_in      IN VARCHAR2,
                                   completed_at_in             IN DATE,
                                   completed_successfully_in   IN VARCHAR2,
                                   ignore_failures_in          IN VARCHAR2)
      RETURN BOOLEAN
   IS
      l_overdue       BOOLEAN := FALSE;
      l_check_again   BOOLEAN := FALSE;
      l_check         BOOLEAN := FALSE;

      FUNCTION is_overdue (freq_in IN VARCHAR2)
         RETURN BOOLEAN
      IS
      BEGIN
         RETURN CASE freq_in
                   WHEN c_freq_never THEN FALSE
                   WHEN c_freq_hourly THEN SYSDATE - completed_at_in >= 1 / 24
                   WHEN c_freq_daily THEN SYSDATE - completed_at_in >= 1
                   ELSE SYSDATE - completed_at_in >= TO_NUMBER (freq_in/(60*24)))
                END;
      END;
   BEGIN
      IF completed_at_in IS NULL
      THEN
         /* Never been run before, so do it the first time. */
         l_check := TRUE;
      ELSE
         /* Might check say every 10 minutes, but you don't want to recur any more frequently than daily. */
         /* If did not complete successfully last time and ignore failures, then see if it should be run again */
         l_overdue := is_overdue (check_frequency_in);
         trc ('check', l_overdue);
         trc ('recur', is_overdue (recurring_frequency_in));

         l_check_again :=
                is_overdue (recurring_frequency_in)
            AND (   completed_successfully_in = 'Y'
                 OR (    completed_successfully_in = 'N'
                     AND ignore_failures_in = 'Y'));

         l_check := l_overdue AND l_check_again;
      END IF;

      RETURN l_check;
   END;

   PROCEDURE execute_alert_actions (alert_id_in IN NUMBER)
   IS
      c_start   CONSTANT DATE := SYSDATE;
      l_alert            alerts%ROWTYPE := one_alert (alert_id_in);
      l_success          BOOLEAN := FALSE;
      l_values           DBMS_SQL.varchar2_table;
      l_code             VARCHAR2 (32767) := l_alert.trigger_code;

      PROCEDURE take_action (alert_action_in IN alert_actions%ROWTYPE)
      IS
         l_send_email_proc_name   alert_config.send_email_proc_name%TYPE;
         l_subject                alert_actions.email_subject%TYPE;
         l_body                   alert_actions.email_body%TYPE;
         l_cursor                 SYS_REFCURSOR;

         PROCEDURE dynamic_send_email (to_in        IN VARCHAR2,
                                       subject_in   IN VARCHAR2,
                                       body_in      IN VARCHAR2)
         IS
         BEGIN
            EXECUTE IMMEDIATE
                  'BEGIN '
               || l_send_email_proc_name
               || '(to_in => :to, subject_in => :sub, body_in => :bod); END;'
               USING to_in, subject_in, body_in;
         END;
      BEGIN
         alert_mgr.LOG ('Take action ' || alert_action_in.action_type);

         CASE alert_action_in.action_type
            WHEN c_action_type_email
            THEN
               SELECT send_email_proc_name
                 INTO l_send_email_proc_name
                 FROM alert_config ac;

               /* Two ways to specify send-to for the emails:
                  - alert send_email_to list
                  - alert_follows
               */

               /* IF there is no email query, then send one. Else.... */
               IF alert_action_in.email_query IS NULL
               THEN
                  dynamic_send_email (alert_action_in.send_email_to,
                                      alert_action_in.email_subject,
                                      alert_action_in.email_body);

                  FOR rec
                     IN (SELECT email_address
                           FROM alert_follows f, alert_users u
                          WHERE f.user_id = u.id AND f.alert_id = alert_id_in)
                  LOOP
                     dynamic_send_email (rec.email_address,
                                         alert_action_in.email_subject,
                                         alert_action_in.email_body);
                  END LOOP;
               ELSE
                  OPEN l_cursor FOR alert_action_in.email_query;

                  LOOP
                     FETCH l_cursor INTO l_subject, l_body;

                     EXIT WHEN l_cursor%NOTFOUND;

                     dynamic_send_email (alert_action_in.send_email_to,
                                         l_subject,
                                         l_body);

                     FOR rec
                        IN (SELECT email_address
                              FROM alert_follows f, alert_users u
                             WHERE     f.user_id = u.id
                                   AND f.alert_id = alert_id_in)
                     LOOP
                        dynamic_send_email (rec.email_address,
                                            l_subject,
                                            l_body);
                     END LOOP;
                  END LOOP;

                  CLOSE l_cursor;
               END IF;
            WHEN c_action_type_proc
            THEN
               EXECUTE IMMEDIATE
                     'BEGIN '
                  || RTRIM (alert_action_in.action_code, ';')
                  || '; end;';
         END CASE;
      END;
   BEGIN
      BEGIN
         CASE
            WHEN l_alert.trigger_type IN (c_trigtype_select_one_row,
                                          c_trigtype_select_no_rows,
                                          c_trigtype_select_mult_rows)
            THEN
               EXECUTE IMMEDIATE l_alert.trigger_code
                  BULK COLLECT INTO l_values;

               l_success :=
                  CASE
                     WHEN l_alert.trigger_type IN (c_trigtype_select_one_row,
                                                   c_trigtype_select_mult_rows)
                     THEN
                           l_values.COUNT = 1
                        OR (    l_alert.trigger_type =
                                   c_trigtype_select_mult_rows
                            AND l_values.COUNT > 1)
                     WHEN l_alert.trigger_type = c_trigtype_select_no_rows
                     THEN
                        l_values.COUNT = 0
                  END;
            WHEN l_alert.trigger_type = c_trigtype_return_true
            THEN
               l_code :=
                     'begin :b := '
                  || RTRIM (l_alert.trigger_code, ';')
                  || '; end;';

               EXECUTE IMMEDIATE l_code USING OUT l_success;
            WHEN l_alert.trigger_type = c_trigtype_return_false
            THEN
               l_code :=
                     'begin :b := '
                  || RTRIM (l_alert.trigger_code, ';')
                  || '; end;';

               EXECUTE IMMEDIATE l_code USING OUT l_success;

               l_success := NOT l_success;
         END CASE;
      EXCEPTION
         WHEN OTHERS
         THEN
            record_result (
               alert_id_in                 => alert_id_in,
               completed_at_in             => SYSDATE,
               completed_successfully_in   => 'N',
               results_in                  =>    'Error executing trigger code: '
                                              || DBMS_UTILITY.format_error_stack
                                              || ' Backtrace: '
                                              || DBMS_UTILITY.format_error_backtrace);
            RETURN;
      END;

      IF l_success
      THEN
         FOR this_action IN (  SELECT *
                                 FROM alert_actions
                                WHERE alert_id = alert_id_in
                             ORDER BY action_sequence)
         LOOP
            take_action (this_action);
         END LOOP;
      END IF;

      record_result (
         alert_id_in                 => alert_id_in,
         completed_at_in             => SYSDATE,
         completed_successfully_in   => 'Y',
         results_in                  => 'Trigger code executed: ' || l_code,
         errors_in                   => NULL);
   EXCEPTION
      WHEN OTHERS
      THEN
         record_result (
            alert_id_in                 => alert_id_in,
            completed_at_in             => SYSDATE,
            completed_successfully_in   => 'N',
            results_in                  => 'Trigger code ailed: ' || l_code);
   END;

   PROCEDURE check_alerts
   IS
   BEGIN
      IF checking_for_alerts
      THEN
         FOR rec IN (SELECT a.*, ar.completed_at, ar.completed_successfully
                       FROM alerts a, alert_runs ar
                      WHERE a.is_active = 'Y' AND a.last_run_id = ar.id(+))
         LOOP
            /* Is the last run more recent than the recurring frequency?
               If so, do nothing. */
            IF time_to_trigger_alert (rec.check_frequency,
                                      rec.recurrence_frequency,
                                      rec.completed_at,
                                      rec.completed_successfully,
                                      rec.ignore_failures)
            THEN
               execute_alert_actions (rec.id);
            END IF;
         END LOOP;
      END IF;
   END;

   /* Add to the alert runs tables and also update the alert row with
         the last run id */

   PROCEDURE record_result (
      alert_id_in                 IN NUMBER,
      completed_at_in             IN DATE DEFAULT SYSDATE,
      completed_successfully_in   IN VARCHAR2 DEFAULT 'Y',
      results_in                  IN VARCHAR2 DEFAULT NULL,
      errors_in                   IN VARCHAR2 DEFAULT NULL)
   IS
      l_id   NUMBER;
   BEGIN
      INSERT INTO alert_runs (alert_id,
                              completed_at,
                              completed_successfully,
                              results,
                              errors)
              VALUES (
                        alert_id_in,
                        completed_at_in,
                        completed_successfully_in,
                        results_in,
                        CASE
                           WHEN completed_successfully_in = 'Y'
                           THEN
                              NULL
                           ELSE
                              NVL (
                                 errors_in,
                                    'Error: '
                                 || DBMS_UTILITY.format_error_stack
                                 || ' Backtrace: '
                                 || DBMS_UTILITY.format_error_backtrace)
                        END)
        RETURNING id
             INTO l_id;

      UPDATE alerts
         SET last_run_id = l_id
       WHERE id = alert_id_in;
   END;

   FUNCTION last_run_for (alert_id_in IN NUMBER)
      RETURN alert_runs%ROWTYPE
   IS
      l_return   alert_runs%ROWTYPE;
   BEGIN
      SELECT ar.*
        INTO l_return
        FROM alert_runs ar, alerts a
       WHERE a.id = alert_id_in AND a.last_run_id = ar.id;

      RETURN l_return;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
   END;

   FUNCTION one_alert (id_in IN NUMBER)
      RETURN alerts%ROWTYPE
   IS
      l_alert   alerts%ROWTYPE;
   BEGIN
      SELECT *
        INTO l_alert
        FROM alerts
       WHERE id = id_in;

      RETURN l_alert;
   END;
END;
/