BEGIN
   alert_mgr.start_check_alerts_job (every_n_minutes_in => 1);
END;
/

BEGIN
   alert_mgr.register_alert (
      alert_name_in             => 'Always True',
      description_in            => 'Trigger is TRUE, display string',
      trigger_code_in           => 'TRUE',
      trigger_type_in           => 'RETURNTRUE',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => 'D',
      ignore_failures_in        => 'Y',
      action_type_in            => 'PROC',
      action_code_in            => 'alert_mgr.log(''PROC EXECUTED'')',
      action_email_in           => '',
      is_active_in              => 'Y');

   alert_mgr.register_alert (
      alert_name_in             => 'Send Email',
      description_in            => 'Trigger is TRUE, send email',
      trigger_code_in           => 'TRUE',
      trigger_type_in           => 'RETURNTRUE',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => 'D',
      ignore_failures_in        => 'Y',
      action_type_in            => 'EMAIL',
      action_code_in            => NULL,
      action_email_in           => 'steven.feuerstein@oracle.com',
      is_active_in              => 'Y');

   alert_mgr.register_alert (
      alert_name_in             => 'Check one row',
      description_in            => 'Always returns one row, no more than once an hour',
      trigger_code_in           => 'select count(*) from dual',
      trigger_type_in           => 'ONEROW',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => 'H',
      ignore_failures_in        => 'Y',
      action_type_in            => 'PROC',
      action_code_in            => 'alert_mgr.log(''One row found'')',
      is_active_in              => 'Y');

   alert_mgr.register_alert (
      alert_name_in             => 'Check no rows - valid',
      description_in            => 'Always returns no rows, no more than every 3 hours',
      trigger_code_in           => 'select count(*) from dual where 1 = 2',
      trigger_type_in           => 'NOROWS',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => '180',
      ignore_failures_in        => 'Y',
      action_type_in            => 'PROC',
      action_code_in            => 'alert_mgr.log(''No rows found'')',
      is_active_in              => 'Y');

   alert_mgr.register_alert (
      alert_name_in             => 'Check one row none returned',
      description_in            => 'Always returns no rows',
      trigger_code_in           => 'select count(*) from dual where 1 = 2',
      trigger_type_in           => 'ONEROW',
      check_frequency_in        => 'H',
      recurrence_frequency_in   => 'D',
      ignore_failures_in        => 'Y',
      action_type_in            => 'PROC',
      action_code_in            => 'alert_mgr.log(''No rows found - Should NOT see this'')',
      is_active_in              => 'Y');
END;
/

/* Use stop_checking to disable alert checking even if and when the job wakes up */

BEGIN
  --alert_mgr.stop_checking;
  alert_mgr.resume_checking;
END;
/

/* Check job details */

SELECT *
  FROM user_scheduler_jobs
 WHERE job_name = 'CHECK_ALERTS_JOB'
/

DECLARE
   d    DATE;
   s    VARCHAR2 (1);
   ig   VARCHAR2 (1);
BEGIN
   alert_mgr.trc (FALSE);
   alert_mgr.bpl (alert_mgr.time_to_trigger_alert (
                     check_frequency_in          => 'H',
                     recurring_frequency_in      => 'D',
                     completed_at_in             => SYSDATE - 1 / 2,
                     completed_successfully_in   => 'Y',
                     ignore_failures_in          => 'Y'));
END;
/

BEGIN
   alert_mgr.bpl (alert_mgr.checking_for_alerts);
END;
/

alter session set nls_date_format='YYYY-MM-DD HH24:MI:SS'
/                      

SELECT * FROM alert_config
/

  SELECT *
    FROM alerts
ORDER BY created DESC
/

  SELECT *
    FROM alert_log
ORDER BY created DESC
/

  SELECT *
    FROM alert_runs
ORDER BY created DESC
/

  SELECT *
    FROM apex_mail_log
ORDER BY last_updated_on DESC;
/

  SELECT *
    FROM apex_mail_queue
ORDER BY last_updated_on DESC;
/

DELETE FROM alert_runs;

DELETE FROM alert_log;